var __require = /* @__PURE__ */ ((x) => typeof require !== "undefined" ? require : typeof Proxy !== "undefined" ? new Proxy(x, {
  get: (a, b) => (typeof require !== "undefined" ? require : a)[b]
}) : x)(function(x) {
  if (typeof require !== "undefined") return require.apply(this, arguments);
  throw Error('Dynamic require of "' + x + '" is not supported');
});

// src/index.ts
import MagicString3 from "magic-string";

// src/analyze.ts
import {
  isArrayExpression,
  isArrayPattern,
  isBlockStatement,
  isBooleanLiteral,
  isClassDeclaration,
  isDeclaration,
  isExportDefaultDeclaration as isExportDefaultDeclaration2,
  isFunctionDeclaration as isFunctionDeclaration2,
  isIdentifier as isIdentifier2,
  isImportDefaultSpecifier,
  isImportNamespaceSpecifier,
  isImportSpecifier,
  isObjectPattern,
  isStringLiteral as isStringLiteral2,
  isTaggedTemplateExpression as isTaggedTemplateExpression2,
  isTemplateLiteral,
  isTSMethodSignature,
  isTSPropertySignature,
  isTSTypeLiteral,
  isVariableDeclaration as isVariableDeclaration2,
  isVariableDeclarator as isVariableDeclarator2
} from "@babel/types";
import { walkIdentifiers as walkIdentifiers2 } from "@vue/compiler-dom";
import hashId from "hash-sum";

// src/babel-helpers/ast.ts
import {
  isArrowFunctionExpression,
  isCallExpression,
  isExportDefaultDeclaration,
  isExportNamedDeclaration,
  isFunctionDeclaration,
  isFunctionExpression,
  isIdentifier,
  isImportDeclaration,
  isMemberExpression,
  isNode,
  isReturnStatement,
  isStringLiteral,
  isTaggedTemplateExpression,
  isVariableDeclaration,
  isVariableDeclarator,
  traverse as traverse2
} from "@babel/types";

// src/constants.ts
var DEFINE_COMPONENT_HELPER = "defineComponent";
var USE_DEFAULTS_HELPER = "useDefaults";
var TO_REFS_HELPER = "toRefs";
var USE_MODEL_HELPER = "useModel";
var CSS_VARS_HELPER = "useCssVars";
var USE_SLOT_HELPER = "useSlots";
var UN_REF_HELPER = "unref";
var DEFAULT_MODEL_MODIFIERS_NAME = "modelModifiers";
var WITH_ASYNC_CONTEXT_HELPER = "withAsyncContext";
var BARE_CALL_MACROS = [
  "vineExpose",
  "vineOptions",
  "vineStyle",
  "vineStyle.scoped",
  "vineStyle.import",
  "vineCustomElement"
];
var VINE_MACROS = [
  "vineProp",
  "vineProp.optional",
  "vineProp.withDefault",
  "vineEmits",
  "vineSlots",
  "vineModel",
  ...BARE_CALL_MACROS
];
var CAN_BE_CALLED_MULTI_TIMES_MACROS = [
  "vineModel",
  "vineStyle"
];
var SUPPORTED_CSS_LANGS = ["css", "scss", "sass", "less", "stylus", "postcss"];
var SUPPORTED_STYLE_FILE_EXTS = [
  ".css",
  ".scss",
  ".sass",
  ".less",
  ".styl"
];
var TS_NODE_TYPES = [
  "TSAsExpression",
  // foo as number
  "TSTypeAssertion",
  // (<number>foo)
  "TSNonNullExpression",
  // foo!
  "TSInstantiationExpression",
  // foo<string>
  "TSSatisfiesExpression"
  // foo satisfies T
];
var VineBindingTypes = {
  /**
   * declared as a prop
   */
  PROPS: "props",
  /**
   * a local alias of a `<script setup>` destructured prop.
   * the original is stored in __propsAliases of the bindingMetadata object.
   */
  PROPS_ALIASED: "props-aliased",
  /**
   * a let binding (may or may not be a ref)
   */
  SETUP_LET: "setup-let",
  /**
   * a const binding that can never be a ref.
   * these bindings don't need `unref()` calls when processed in inlined
   * template expressions.
   */
  SETUP_CONST: "setup-const",
  /**
   * a const binding that does not need `unref()`, but may be mutated.
   */
  SETUP_REACTIVE_CONST: "setup-reactive-const",
  /**
   * a const binding that may be a ref.
   */
  SETUP_MAYBE_REF: "setup-maybe-ref",
  /**
   * bindings that are guaranteed to be refs
   */
  SETUP_REF: "setup-ref",
  /**
   * a literal constant, e.g. 'foo', 1, true
   */
  LITERAL_CONST: "literal-const"
};
var EXPECTED_ERROR = "expected_error";

// src/utils/index.ts
import { traverse } from "@babel/types";
var camelizeRE = /-(\w)/g;
function cacheStringFunction(fn) {
  const cache = /* @__PURE__ */ Object.create(null);
  return (str) => {
    const hit = cache[str];
    return hit || (cache[str] = fn(str));
  };
}
function showIf(condition, s, not) {
  return condition ? s : not ?? "";
}
function filterJoin(arr, join) {
  return arr.filter(Boolean).join(join);
}
function getStyleLangFileExt(styleLang) {
  if (styleLang === "postcss") {
    return "css";
  } else if (styleLang === "stylus") {
    return "styl";
  }
  return styleLang;
}
function appendToMapArray(storeMap, key, value) {
  const arr = storeMap.get(key);
  if (!arr) {
    storeMap.set(key, [value]);
  } else {
    arr.push(value);
  }
}
var camelize = cacheStringFunction((str) => {
  return str.replace(camelizeRE, (_, c) => c ? c.toUpperCase() : "");
});
var capitalize = cacheStringFunction(
  (str) => str.charAt(0).toUpperCase() + str.slice(1)
);
var ExitTraverseError = class extends Error {
  constructor() {
    super("ExitTraverse");
  }
};
var exitTraverse = new ExitTraverseError();
var _breakableTraverse = (node, handlers) => {
  try {
    return traverse(node, handlers);
  } catch (e) {
    if (e instanceof ExitTraverseError) {
      return;
    }
    throw e;
  }
};

// src/babel-helpers/ast.ts
function isVineCompFnDecl(target) {
  if ((isExportNamedDeclaration(target) || isExportDefaultDeclaration(target)) && target.declaration) {
    target = target.declaration;
  }
  if (isFunctionDeclaration(target) || isVariableDeclaration(target)) {
    try {
      traverse2(target, (node) => {
        if (isReturnStatement(node) && node.argument && isVineTaggedTemplateString(node.argument)) {
          throw new Error(EXPECTED_ERROR);
        }
        if (isArrowFunctionExpression(node) && isVineTaggedTemplateString(node.body)) {
          throw new Error(EXPECTED_ERROR);
        }
      });
    } catch (error) {
      if (error.message === EXPECTED_ERROR) {
        return true;
      }
      throw error;
    }
  }
  return false;
}
function findVineCompFnDecls(root) {
  const vineFnComps = [];
  for (const stmt of root.program.body) {
    if (isVineCompFnDecl(stmt)) {
      vineFnComps.push(stmt);
    }
  }
  return vineFnComps;
}
function isDescendant(node, potentialDescendant) {
  const stack = [node];
  while (stack.length) {
    const currentNode = stack.pop();
    if (currentNode === potentialDescendant) {
      return true;
    }
    const children = Object.values(currentNode).filter((child) => Array.isArray(child) ? child.every(isNode) : isNode(child));
    for (const child of children) {
      if (Array.isArray(child)) {
        stack.push(...child);
      } else {
        stack.push(child);
      }
    }
  }
  return false;
}
function isVineTaggedTemplateString(node) {
  return isTaggedTemplateExpression(node) && isIdentifier(node.tag) && node.tag.name === "vine";
}
function isVineMacroCallExpression(node) {
  if (isCallExpression(node)) {
    const callee = node.callee;
    if (isIdentifier(callee)) {
      return VINE_MACROS.includes(callee.name);
    }
    if (isMemberExpression(callee)) {
      const obj = callee.object;
      const prop = callee.property;
      if (isIdentifier(obj) && isIdentifier(prop)) {
        return VINE_MACROS.includes(
          `${obj.name}.${prop.name}`
        );
      }
    }
  }
  return false;
}
function getVineMacroCalleeName(node) {
  const callee = node.callee;
  if (isIdentifier(callee)) {
    return callee.name;
  }
  if (isMemberExpression(callee)) {
    const obj = callee.object;
    const prop = callee.property;
    if (isIdentifier(obj) && isIdentifier(prop)) {
      return `${obj.name}.${prop.name}`;
    }
  }
  return "";
}
function getVinePropCallTypeParams(node) {
  return node.typeParameters?.params?.[0];
}
function isVineMacroOf(name) {
  return (node) => {
    if (isCallExpression(node)) {
      const callee = node.callee;
      if (isIdentifier(callee)) {
        return Array.isArray(name) ? name.includes(callee.name) : callee.name === name;
      }
      if (isMemberExpression(callee)) {
        const obj = callee.object;
        const prop = callee.property;
        if (isIdentifier(obj) && isIdentifier(prop)) {
          return Array.isArray(name) ? name.some((n) => `${obj.name}.${prop.name}`.includes(n)) : `${obj.name}.${prop.name}`.includes(name);
        }
        return false;
      }
    }
    return false;
  };
}
var isVineProp = isVineMacroOf("vineProp");
var isVineSlots = isVineMacroOf("vineSlots");
var isVineEmits = isVineMacroOf("vineEmits");
var isVineModel = isVineMacroOf("vineModel");
var isVineStyle = isVineMacroOf("vineStyle");
var isVineCustomElement = isVineMacroOf("vineCustomElement");
function isVineImportScoped(node) {
  if (!isCallExpression(node)) {
    return false;
  }
  const callee = node.callee;
  if (!isMemberExpression(callee)) {
    return false;
  }
  return isVineStyle(callee.object) && isIdentifier(callee.property) && callee.property.name === "scoped";
}
function isStatementContainsVineMacroCall(node) {
  let result = false;
  _breakableTraverse(node, (descendant) => {
    if (isVineMacroCallExpression(descendant)) {
      result = true;
      throw exitTraverse;
    }
  });
  return result;
}
function isTagTemplateStringContainsInterpolation(tagTmplNode) {
  return tagTmplNode.quasi.expressions.length > 0;
}
function getFunctionParams(fnItselfNode) {
  const params = [];
  if (isFunctionDeclaration(fnItselfNode)) {
    params.push(...fnItselfNode.params);
  } else if (isFunctionExpression(fnItselfNode) || isArrowFunctionExpression(fnItselfNode)) {
    params.push(...fnItselfNode.params);
  }
  return params;
}
function getFunctionPickedInfos(fnDecl) {
  const pickedInfo = [];
  let target = fnDecl;
  if ((isExportNamedDeclaration(target) || isExportDefaultDeclaration(target)) && target.declaration) {
    target = target.declaration;
  }
  if (isVariableDeclaration(target)) {
    target.declarations.forEach(
      (decl) => {
        if ((isFunctionExpression(decl.init) || isArrowFunctionExpression(decl.init)) && isIdentifier(decl.id)) {
          pickedInfo.push({
            fnDeclNode: fnDecl,
            fnItselfNode: decl.init,
            fnName: decl.id.name
          });
        }
      }
    );
    return pickedInfo;
  }
  if (isFunctionDeclaration(target)) {
    pickedInfo.push({
      fnDeclNode: fnDecl,
      fnItselfNode: target,
      fnName: target.id?.name ?? ""
    });
  }
  return pickedInfo;
}
function findVineTagTemplateStringReturn(node) {
  let templateReturn;
  let templateStringNode;
  traverse2(node, (descendant) => {
    if (isReturnStatement(descendant)) {
      templateReturn = descendant;
      if (isVineTaggedTemplateString(descendant.argument)) {
        templateStringNode = descendant.argument;
      }
    }
  });
  return {
    templateReturn,
    templateStringNode
  };
}
function getImportStatements(root) {
  const importStmts = [];
  for (const stmt of root.program.body) {
    if (isImportDeclaration(stmt)) {
      importStmts.push(stmt);
    }
  }
  return importStmts;
}
function getTSTypeLiteralPropertySignatureName(tsTypeLit) {
  return isIdentifier(tsTypeLit.key) ? tsTypeLit.key.name : isStringLiteral(tsTypeLit.key) ? tsTypeLit.key.value : "";
}
function getAllVinePropMacroCall(fnItselfNode) {
  const allVinePropMacroCalls = [];
  traverse2(fnItselfNode.body, {
    enter(node, parent) {
      if (!isVineMacroOf("vineProp")(node)) {
        return;
      }
      const propVarDeclarator = parent.find((ancestor) => isVariableDeclarator(ancestor.node) && isIdentifier(ancestor.node.id));
      if (!propVarDeclarator) {
        return;
      }
      const propVarIdentifier = propVarDeclarator.node.id;
      allVinePropMacroCalls.push([node, propVarIdentifier]);
    }
  });
  return allVinePropMacroCalls;
}
function unwrapTSNode(node) {
  if (TS_NODE_TYPES.includes(node.type)) {
    return unwrapTSNode(node.expression);
  } else {
    return node;
  }
}
function isStaticNode(node) {
  node = unwrapTSNode(node);
  switch (node.type) {
    case "UnaryExpression":
      return isStaticNode(node.argument);
    case "LogicalExpression":
    // 1 > 2
    case "BinaryExpression":
      return isStaticNode(node.left) && isStaticNode(node.right);
    case "ConditionalExpression": {
      return isStaticNode(node.test) && isStaticNode(node.consequent) && isStaticNode(node.alternate);
    }
    case "SequenceExpression":
    // (1, 2)
    case "TemplateLiteral":
      return node.expressions.every((expr) => isStaticNode(expr));
    case "ParenthesizedExpression":
      return isStaticNode(node.expression);
    case "StringLiteral":
    case "NumericLiteral":
    case "BooleanLiteral":
    case "NullLiteral":
    case "BigIntLiteral":
      return true;
  }
  return false;
}
function isCallOf(node, test) {
  return !!(node && test && node.type === "CallExpression" && node.callee.type === "Identifier" && (typeof test === "string" ? node.callee.name === test : test(node.callee.name)));
}
function isLiteralNode(node) {
  return node.type.endsWith("Literal");
}
function canNeverBeRef(node, userReactiveImport) {
  if (isCallOf(node, userReactiveImport)) {
    return true;
  }
  switch (node.type) {
    case "UnaryExpression":
    case "BinaryExpression":
    case "ArrayExpression":
    case "ObjectExpression":
    case "FunctionExpression":
    case "ArrowFunctionExpression":
    case "UpdateExpression":
    case "ClassExpression":
    case "TaggedTemplateExpression":
      return true;
    case "SequenceExpression":
      return canNeverBeRef(
        node.expressions[node.expressions.length - 1],
        userReactiveImport
      );
    default:
      if (isLiteralNode(node)) {
        return true;
      }
      return false;
  }
}
function tryInferExpressionTSType(node) {
  switch (node.type) {
    case "BooleanLiteral":
      return "boolean";
    case "NumericLiteral":
      return "number";
    case "BigIntLiteral":
      return "bigint";
    case "StringLiteral":
      return "string";
    case "NullLiteral":
      return "null";
    case "Identifier":
      return node.name === "undefined" ? "undefined" : `typeof ${node.name}`;
    default:
      return "any";
  }
}

// src/utils/color-string.ts
var colorMap = {
  bold: "1",
  italic: "3",
  underline: "4",
  inverse: "7",
  strikethrough: "9",
  white: "37",
  grey: "90",
  black: "30",
  blue: "34",
  cyan: "36",
  green: "32",
  magenta: "35",
  red: "31",
  yellow: "33",
  bgWhite: "47",
  bgGrey: "49",
  bgBlack: "40",
  bgBlue: "44",
  bgCyan: "46",
  bgGreen: "42",
  bgMagenta: "45",
  bgRed: "41",
  bgYellow: "43"
};
function colorful(str, options = []) {
  const colors = options.map((option) => {
    return colorMap[option];
  }).join(";");
  return `\x1B[${colors}m${str}\x1B[0m`;
}
function createColorful(options) {
  return (str) => colorful(str, options);
}
var Colorful = Object.keys(colorMap).reduce((acc, key) => {
  acc[key] = createColorful([key]);
  return acc;
}, {});

// src/diagnostics.ts
var whiteFgRedBg = createColorful(["white", "bgRed"]);
var whiteFgYellowBg = createColorful(["white", "bgYellow"]);
var redFgBold = createColorful(["red", "bold"]);
var ErrorLabel = whiteFgRedBg(" Error ");
var WarningLabel = whiteFgYellowBg(" Warning ");
function makeDiagnostic(label, { msg, fileId, location, msgColorful }) {
  const pos = location?.start;
  const posString = pos ? `${pos.line}:${pos.column}` : "";
  return `
${label} File: ${fileId} ${posString}
${msgColorful(msg)}
`;
}
function vineErr({ vineFileCtx, vineCompFnCtx }, { msg, location, rawVueTemplateLocation }) {
  const full = makeDiagnostic(ErrorLabel, {
    msg,
    location,
    fileId: vineFileCtx.fileId,
    msgColorful: redFgBold
  });
  return { full, msg, location, rawVueTemplateLocation, vineCompFnCtx };
}
function vineWarn({ vineFileCtx, vineCompFnCtx }, { msg, location, rawVueTemplateLocation }) {
  const full = makeDiagnostic(WarningLabel, {
    msg,
    location,
    fileId: vineFileCtx.fileId,
    msgColorful: whiteFgYellowBg
  });
  return { full, msg, location, rawVueTemplateLocation, vineCompFnCtx };
}

// src/style/analyze-css-vars.ts
function lexBinding(content, start) {
  let state = 1 /* inParens */;
  let parenDepth = 0;
  for (let i = start; i < content.length; i++) {
    const char = content.charAt(i);
    switch (state) {
      case 1 /* inParens */:
        if (char === "'") {
          state = 2 /* inSingleQuoteString */;
        } else if (char === '"') {
          state = 3 /* inDoubleQuoteString */;
        } else if (char === "(") {
          parenDepth++;
        } else if (char === ")") {
          if (parenDepth > 0) {
            parenDepth--;
          } else {
            return i;
          }
        }
        break;
      case 2 /* inSingleQuoteString */:
        if (char === "'") {
          state = 1 /* inParens */;
        }
        break;
      case 3 /* inDoubleQuoteString */:
        if (char === '"') {
          state = 1 /* inParens */;
        }
        break;
    }
  }
  return null;
}
function normalizeExpression(exp) {
  exp = exp.trim();
  if (exp[0] === "'" && exp[exp.length - 1] === "'" || exp[0] === '"' && exp[exp.length - 1] === '"') {
    return exp.slice(1, -1);
  }
  return exp;
}
var vBindRE = /v-bind\s*\(/g;
function parseCssVars(styles, hook) {
  const vars = [];
  styles.forEach((style) => {
    let match = null;
    const content = style.replace(/\/\*([\s\S]*?)\*\//g, "");
    for (match = vBindRE.exec(content); match; match = vBindRE.exec(content)) {
      const start = match.index + match[0].length;
      const end = lexBinding(content, start);
      if (end !== null) {
        hook?.getIndex(start, end);
        const variable = normalizeExpression(content.slice(start, end));
        if (!vars.includes(variable)) {
          vars.push(variable);
        }
      }
    }
  });
  return vars;
}

// src/template/import-usage-check.ts
import {
  NodeTypes,
  parserOptions,
  walkIdentifiers
} from "@vue/compiler-dom";
import { camelize as camelize2, capitalize as capitalize2, isBuiltInDirective } from "@vue/shared";

// src/template/cache.ts
import { LRUCache } from "lru-cache";
function createCache(max = 500) {
  return new LRUCache({ max });
}

// src/template/import-usage-check.ts
var templateUsageCheckCache = createCache();
function isImportUsed(vineCompFnCtx, local) {
  return resolveTemplateUsedIdentifiers(vineCompFnCtx).has(local);
}
function resolveTemplateUsedIdentifiers(vineCompFnCtx) {
  const { templateSource, templateParsedAst } = vineCompFnCtx;
  const cached = templateUsageCheckCache.get(templateSource);
  if (cached) {
    return cached;
  }
  const ids = /* @__PURE__ */ new Set();
  templateParsedAst.children.forEach(walk);
  function walk(node) {
    switch (node.type) {
      case NodeTypes.ELEMENT: {
        let tag = node.tag;
        if (tag.includes("."))
          tag = tag.split(".")[0].trim();
        if (!parserOptions.isNativeTag(tag) && !parserOptions.isBuiltInComponent(tag)) {
          ids.add(camelize2(tag));
          ids.add(capitalize2(camelize2(tag)));
        }
        for (let i = 0; i < node.props.length; i++) {
          const prop = node.props[i];
          if (prop.type === NodeTypes.DIRECTIVE) {
            if (!isBuiltInDirective(prop.name)) {
              ids.add(`v${capitalize2(camelize2(prop.name))}`);
            }
            if (prop.arg && !prop.arg.isStatic) {
              extractIdentifiers(ids, prop.arg);
            }
            if (prop.name === "for") {
              extractIdentifiers(ids, prop.forParseResult.source);
            } else if (prop.exp) {
              extractIdentifiers(ids, prop.exp);
            } else if (prop.name === "bind" && !prop.exp) {
              ids.add(camelize2(prop.arg.content));
            }
          }
          if (prop.type === NodeTypes.ATTRIBUTE && prop.name === "ref" && prop.value?.content) {
            ids.add(prop.value.content);
          }
        }
        node.children.forEach(walk);
        break;
      }
      case NodeTypes.INTERPOLATION:
        extractIdentifiers(ids, node.content);
        break;
    }
  }
  templateUsageCheckCache.set(templateSource, ids);
  return ids;
}
function extractIdentifiers(ids, node) {
  if (node.ast) {
    walkIdentifiers(node.ast, (n) => ids.add(n.name));
  } else if (node.ast === null) {
    ids.add(node.content);
  }
}

// src/analyze.ts
function storeTheOnlyMacroCallArg(macroName, callback) {
  const findMacroCallNode = (fnItselfNode) => {
    let vineExposeMacroCall;
    _breakableTraverse(fnItselfNode, {
      enter(descendant) {
        if (isVineMacroOf(macroName)(descendant)) {
          vineExposeMacroCall = descendant;
          throw exitTraverse;
        }
      }
    });
    return vineExposeMacroCall;
  };
  return (analyzeCtx, fnItselfNode) => {
    const macroCall = findMacroCallNode(fnItselfNode);
    if (!macroCall) {
      return;
    }
    const macroCallArg = macroCall.arguments[0];
    if (!macroCallArg) {
      return;
    }
    callback(analyzeCtx, macroCallArg);
  };
}
var analyzeVineExpose = storeTheOnlyMacroCallArg(
  "vineExpose",
  ({ vineCompFnCtx }, macroCallArg) => {
    vineCompFnCtx.expose = macroCallArg;
  }
);
var analyzeVineOptions = storeTheOnlyMacroCallArg(
  "vineOptions",
  ({ vineCompFnCtx }, macroCallArg) => {
    vineCompFnCtx.options = macroCallArg;
  }
);
function registerBinding(bindings, node, type) {
  bindings[node.name] = type;
}
function walkPattern(node, bindings, isConst) {
  if (node.type === "Identifier") {
    const type = isConst ? VineBindingTypes.SETUP_MAYBE_REF : VineBindingTypes.SETUP_LET;
    registerBinding(bindings, node, type);
  } else if (node.type === "RestElement") {
    const type = isConst ? VineBindingTypes.SETUP_CONST : VineBindingTypes.SETUP_LET;
    registerBinding(bindings, node.argument, type);
  } else if (node.type === "ObjectPattern") {
    walkObjectPattern(node, bindings, isConst);
  } else if (node.type === "ArrayPattern") {
    walkArrayPattern(node, bindings, isConst);
  } else if (node.type === "AssignmentPattern") {
    if (node.left.type === "Identifier") {
      const type = isConst ? VineBindingTypes.SETUP_MAYBE_REF : VineBindingTypes.SETUP_LET;
      registerBinding(bindings, node.left, type);
    } else {
      walkPattern(node.left, bindings, isConst);
    }
  }
}
function walkArrayPattern(node, bindings, isConst) {
  for (const e of node.elements) {
    if (!e)
      continue;
    walkPattern(e, bindings, isConst);
  }
}
function walkObjectPattern(node, bindings, isConst) {
  for (const p of node.properties) {
    if (p.type === "ObjectProperty") {
      if (p.key.type === "Identifier" && p.key === p.value) {
        const type = isConst ? VineBindingTypes.SETUP_MAYBE_REF : VineBindingTypes.SETUP_LET;
        registerBinding(bindings, p.key, type);
      } else {
        walkPattern(p.value, bindings, isConst);
      }
    } else {
      const type = isConst ? VineBindingTypes.SETUP_CONST : VineBindingTypes.SETUP_LET;
      registerBinding(bindings, p.argument, type);
    }
  }
}
function analyzeVariableDeclarationForBindings({ vineFileCtx, vineCompFnCtx }, stmt) {
  const userImportAliases = { ...vineFileCtx.vueImportAliases };
  const userReactiveBinding = userImportAliases.reactive;
  const allDeclarators = stmt.declarations;
  const isConst = stmt.kind === "const";
  const isAllLiteral = isConst && allDeclarators.every(
    (decl) => isIdentifier2(decl.id) && decl.init && isStaticNode(decl.init)
  );
  for (const varDecl of allDeclarators) {
    if (isIdentifier2(varDecl.id) && varDecl.init) {
      let bindingType;
      if (isAllLiteral || isConst && isStaticNode(varDecl.init)) {
        bindingType = VineBindingTypes.LITERAL_CONST;
      } else if (isCallOf(varDecl.init, userReactiveBinding)) {
        bindingType = isConst ? VineBindingTypes.SETUP_REACTIVE_CONST : VineBindingTypes.SETUP_LET;
      } else if (
        // if a declaration is a const literal, we can mark it so that
        // the generated render fn code doesn't need to unref() it
        isConst && canNeverBeRef(varDecl.init, userReactiveBinding)
      ) {
        bindingType = VineBindingTypes.SETUP_CONST;
      } else if (isConst) {
        if (isCallOf(
          varDecl.init,
          (m) => m === userImportAliases.ref || m === userImportAliases.computed || m === userImportAliases.shallowRef || m === userImportAliases.customRef || m === userImportAliases.toRef
        )) {
          bindingType = VineBindingTypes.SETUP_REF;
        } else {
          bindingType = VineBindingTypes.SETUP_MAYBE_REF;
        }
      } else {
        bindingType = VineBindingTypes.SETUP_LET;
      }
      registerBinding(vineCompFnCtx.bindings, varDecl.id, bindingType);
    } else if (isObjectPattern(varDecl.id)) {
      walkObjectPattern(
        varDecl.id,
        vineCompFnCtx.bindings,
        isConst
      );
    } else if (isArrayPattern(varDecl.id)) {
      walkArrayPattern(
        varDecl.id,
        vineCompFnCtx.bindings,
        isConst
      );
    }
  }
  return isAllLiteral;
}
function analyzeVineFnBodyStmtForBindings(analyzeCtx, stmt) {
  const { vineCompFnCtx } = analyzeCtx;
  let isAllLiteral = false;
  switch (stmt.type) {
    case "VariableDeclaration":
      isAllLiteral = analyzeVariableDeclarationForBindings(analyzeCtx, stmt);
      break;
    case "TSEnumDeclaration":
      isAllLiteral = stmt.members.every(
        (member) => !member.initializer || isStaticNode(member.initializer)
      );
      vineCompFnCtx.bindings[stmt.id.name] = isAllLiteral ? VineBindingTypes.LITERAL_CONST : VineBindingTypes.SETUP_CONST;
      break;
    case "FunctionDeclaration":
    case "ClassDeclaration":
      vineCompFnCtx.bindings[stmt.id.name] = VineBindingTypes.SETUP_CONST;
      break;
    default:
      break;
  }
  return isAllLiteral;
}
function getVineStyleSource(vineStyleArg) {
  let styleLang = "css";
  let styleSource = "";
  let range;
  if (isTaggedTemplateExpression2(vineStyleArg)) {
    const { tag } = vineStyleArg;
    if (isIdentifier2(tag)) {
      styleLang = tag.name;
      const styleSourceNode = vineStyleArg.quasi.quasis[0];
      styleSource = styleSourceNode.value.raw;
      range = [styleSourceNode.start, styleSourceNode.end];
    }
  } else if (isStringLiteral2(vineStyleArg)) {
    styleSource = vineStyleArg.value;
    range = [vineStyleArg.start, vineStyleArg.end];
  } else if (isTemplateLiteral(vineStyleArg)) {
    const styleSourceNode = vineStyleArg.quasis[0];
    styleSource = styleSourceNode.value.raw;
    range = [styleSourceNode.start, styleSourceNode.end];
  }
  return {
    styleSource,
    styleLang,
    range
  };
}
var analyzeVineProps = ({ vineCompilerHooks, vineCompFnCtx, vineFileCtx }, fnItselfNode) => {
  const formalParams = getFunctionParams(fnItselfNode);
  if (formalParams.length === 1) {
    const propsFormalParam = formalParams[0];
    const propsTypeAnnotation = propsFormalParam.typeAnnotation?.typeAnnotation;
    if (!propsTypeAnnotation) {
      return;
    }
    vineCompFnCtx.propsFormalParam = propsTypeAnnotation;
    vineCompFnCtx.propsAlias = propsFormalParam.name;
    propsTypeAnnotation.members?.forEach((member) => {
      if (!isIdentifier2(member.key) || !member.typeAnnotation) {
        return;
      }
      const propName = member.key.name;
      const propType = vineFileCtx.getAstNodeContent(member.typeAnnotation.typeAnnotation);
      const propMeta = {
        isFromMacroDefine: false,
        isRequired: member.optional === void 0 ? true : !member.optional,
        isBool: [
          "boolean",
          "Boolean",
          "true",
          "false"
        ].includes(propType),
        typeAnnotationRaw: propType
      };
      vineCompFnCtx.props[propName] = propMeta;
      vineCompFnCtx.bindings[propName] = VineBindingTypes.PROPS;
    });
  } else if (formalParams.length === 0) {
    vineCompFnCtx.propsDefinitionBy = "macro";
    const allVinePropMacroCalls = getAllVinePropMacroCall(fnItselfNode);
    allVinePropMacroCalls.forEach(([macroCall, propVarIdentifier]) => {
      const macroCalleeName = getVineMacroCalleeName(macroCall);
      const propMeta = {
        isFromMacroDefine: true,
        isRequired: macroCalleeName !== "vineProp.optional",
        isBool: false,
        typeAnnotationRaw: "any",
        declaredIdentifier: propVarIdentifier
      };
      if (macroCalleeName === "vineProp.withDefault") {
        propMeta.default = macroCall.arguments[0];
        propMeta.validator = macroCall.arguments[1];
        propMeta.isBool = isBooleanLiteral(propMeta.default);
        propMeta.isRequired = false;
        const inferredType = tryInferExpressionTSType(propMeta.default);
        propMeta.typeAnnotationRaw = inferredType;
      } else {
        propMeta.validator = macroCall.arguments[0];
      }
      const macroCallTypeParamNode = getVinePropCallTypeParams(macroCall);
      if (macroCallTypeParamNode) {
        const macroCallTypeParam = vineFileCtx.getAstNodeContent(macroCallTypeParamNode);
        propMeta.isBool = macroCallTypeParam === "boolean";
        propMeta.typeAnnotationRaw = macroCallTypeParam;
      }
      if (propMeta.typeAnnotationRaw === "any") {
        vineCompilerHooks.onWarn(
          vineWarn({ vineFileCtx, vineCompFnCtx }, {
            msg: "The default value is an expression, Vine compiler doesn't embed TypeScript to infer its type. So it's recommended to provide a type anonation explicitly for IDE checking.",
            location: macroCall.loc
          })
        );
      }
      const propName = propVarIdentifier.name;
      vineCompFnCtx.props[propName] = propMeta;
      vineCompFnCtx.bindings[propName] = VineBindingTypes.SETUP_REF;
      vineCompFnCtx.linkedMacroCalls.push({
        macroCall,
        macroType: "vineProp",
        macroMeta: propMeta
      });
    });
  }
};
var analyzeVineEmits = (analyzeCtx, fnItselfNode) => {
  const { vineCompFnCtx } = analyzeCtx;
  let vineEmitsMacroCall;
  let parentVarDecl;
  _breakableTraverse(fnItselfNode, {
    enter(descendant, parent) {
      if (isVineEmits(descendant)) {
        vineEmitsMacroCall = descendant;
        const foundVarDeclAncestor = parent.find((ancestor) => isVariableDeclarator2(ancestor.node));
        parentVarDecl = foundVarDeclAncestor?.node;
        vineCompFnCtx.linkedMacroCalls.push({
          macroType: "vineEmits",
          macroCall: vineEmitsMacroCall
        });
        throw exitTraverse;
      }
    }
  });
  if (!vineEmitsMacroCall) {
    return;
  }
  const typeParam = vineEmitsMacroCall.typeParameters?.params[0];
  const callArg = vineEmitsMacroCall.arguments[0];
  if (typeParam && isTSTypeLiteral(typeParam)) {
    vineCompFnCtx.emitsTypeParam = typeParam;
    const emitsTypeLiteralProps = typeParam.members;
    emitsTypeLiteralProps.forEach((prop) => {
      const propName = getTSTypeLiteralPropertySignatureName(prop);
      vineCompFnCtx.emits.push(propName);
    });
  } else if (isArrayExpression(callArg)) {
    callArg.elements.forEach((el) => {
      if (isStringLiteral2(el)) {
        vineCompFnCtx.emits.push(el.value);
      }
    });
  }
  if (parentVarDecl) {
    vineCompFnCtx.emitsAlias = parentVarDecl.id.name;
  }
};
var analyzeVineBindings = (analyzeCtx, fnItselfNode) => {
  const { vineFileCtx, vineCompFnCtx } = analyzeCtx;
  const notContainsMacroStatements = [];
  const fnBody = fnItselfNode.body;
  if (!isBlockStatement(fnBody)) {
    return;
  }
  for (const stmt of fnBody.body) {
    let hasMacroCall = false;
    _breakableTraverse(stmt, (node) => {
      if (isVineMacroCallExpression(node)) {
        hasMacroCall = true;
        throw exitTraverse;
      }
    });
    if (!hasMacroCall) {
      notContainsMacroStatements.push(stmt);
    }
  }
  for (const stmt of notContainsMacroStatements) {
    const isAllLiteral = analyzeVineFnBodyStmtForBindings(analyzeCtx, stmt);
    if (isAllLiteral) {
      vineCompFnCtx.hoistSetupStmts.push(stmt);
    }
  }
  for (const [importName, {
    isType,
    isNamespace,
    isDefault,
    source
  }] of Object.entries(vineFileCtx.userImports)) {
    if (isType) {
      continue;
    }
    const isSetupConst = isNamespace || isDefault && source.endsWith(".vue") || source === "vue";
    vineCompFnCtx.bindings[importName] = isSetupConst ? VineBindingTypes.SETUP_CONST : VineBindingTypes.SETUP_MAYBE_REF;
  }
  const allTopLevelDeclStmts = vineFileCtx.root.program.body.filter((stmt) => isDeclaration(stmt));
  for (const declStmt of allTopLevelDeclStmts) {
    if (isVineCompFnDecl(declStmt)) {
      const pickedInfos = getFunctionPickedInfos(declStmt);
      pickedInfos.forEach(({ fnName }) => {
        vineCompFnCtx.bindings[fnName] = VineBindingTypes.SETUP_CONST;
      });
    } else if (isVariableDeclaration2(declStmt)) {
      for (const decl of declStmt.declarations) {
        if (isVariableDeclarator2(decl) && isIdentifier2(decl.id)) {
          vineCompFnCtx.bindings[decl.id.name] = VineBindingTypes.LITERAL_CONST;
        }
      }
    } else if ((isFunctionDeclaration2(declStmt) || isClassDeclaration(declStmt)) && declStmt.id) {
      vineCompFnCtx.bindings[declStmt.id.name] = VineBindingTypes.LITERAL_CONST;
    }
  }
};
var analyzeVineStyle = ({ vineFileCtx, vineCompFnCtx, vineCompilerHooks }, fnItselfNode) => {
  let vineStyleMacroCalls = [];
  let vineStyleMacroCallOfScopedVineStyleImport = /* @__PURE__ */ new WeakSet();
  _breakableTraverse(fnItselfNode, (node) => {
    if (isVineImportScoped(node)) {
      vineStyleMacroCallOfScopedVineStyleImport.add(
        node.callee.object
      );
    } else if (isVineStyle(node)) {
      vineStyleMacroCalls.push(node);
    }
  });
  if (!vineStyleMacroCalls.length) {
    return;
  }
  for (const vineStyleMacroCall of vineStyleMacroCalls) {
    const macroCalleeName = getVineMacroCalleeName(vineStyleMacroCall);
    const vineStyleArg = vineStyleMacroCall.arguments[0];
    if (!vineStyleArg) {
      return;
    }
    const {
      styleLang,
      styleSource,
      range
    } = getVineStyleSource(
      vineStyleArg
    );
    const isExternalFilePathSource = macroCalleeName === "vineStyle.import";
    const styleMeta = {
      lang: styleLang,
      source: styleSource,
      isExternalFilePathSource,
      range,
      scoped: macroCalleeName === "vineStyle.scoped" || macroCalleeName === "vineStyle.import" && vineStyleMacroCallOfScopedVineStyleImport.has(vineStyleMacroCall),
      fileCtx: vineFileCtx,
      compCtx: vineCompFnCtx
    };
    if (isExternalFilePathSource) {
      vineCompFnCtx.externalStyleFilePaths.push(styleSource);
      const fileExt = SUPPORTED_STYLE_FILE_EXTS.find((ext) => styleSource.endsWith(ext));
      if (fileExt) {
        styleMeta.lang = fileExt.slice(1);
      } else {
        vineCompilerHooks.onError(
          vineErr({ vineFileCtx, vineCompFnCtx }, {
            msg: "Invalid external style file",
            location: vineStyleArg.loc
          })
        );
      }
    }
    if (vineCompFnCtx.scopeId) {
      vineFileCtx.styleDefine[vineCompFnCtx.scopeId] ??= [];
      vineFileCtx.styleDefine[vineCompFnCtx.scopeId].push(styleMeta);
    }
    if (isExternalFilePathSource) {
      continue;
    }
    const cssvarsValueList = parseCssVars([styleSource]);
    if (cssvarsValueList.length > 0) {
      vineCompFnCtx.cssBindings ??= {};
      cssvarsValueList.forEach((value) => {
        vineCompFnCtx.cssBindings[value] = hashId(`${vineCompFnCtx.fnName}__${value}`);
      });
    }
  }
};
var analyzeVineCustomElement = ({ vineCompFnCtx }, fnItselfNode) => {
  _breakableTraverse(fnItselfNode, (node) => {
    if (isVineCustomElement(node)) {
      vineCompFnCtx.isCustomElement = true;
      throw exitTraverse;
    }
  });
};
var analyzeVineSlots = ({ vineCompFnCtx }, fnItselfNode) => {
  let vineSlotsMacroCall;
  let parentVarDecl;
  _breakableTraverse(fnItselfNode, (node, parent) => {
    if (isVineSlots(node)) {
      vineSlotsMacroCall = node;
      const foundVarDeclAncestor = parent.find((ancestor) => isVariableDeclarator2(ancestor.node));
      parentVarDecl = foundVarDeclAncestor?.node;
      vineCompFnCtx.linkedMacroCalls.push({
        macroType: "vineSlots",
        macroCall: node
      });
      throw exitTraverse;
    }
  });
  if (!vineSlotsMacroCall) {
    return;
  }
  const typeParam = vineSlotsMacroCall.typeParameters?.params[0];
  if (!typeParam) {
    return;
  }
  const slotsTypeLiteralProps = typeParam.members;
  for (const prop of slotsTypeLiteralProps) {
    if (isTSPropertySignature(prop) && isIdentifier2(prop.key)) {
      const fnFirstParamType = prop.typeAnnotation?.typeAnnotation?.parameters?.[0]?.typeAnnotation?.typeAnnotation;
      if (fnFirstParamType) {
        vineCompFnCtx.slots[prop.key.name] = {
          props: fnFirstParamType
        };
      }
    } else if (isTSMethodSignature(prop) && isIdentifier2(prop.key)) {
      const fnFirstParamType = prop.parameters[0].typeAnnotation.typeAnnotation;
      vineCompFnCtx.slots[prop.key.name] = {
        props: fnFirstParamType
      };
    }
  }
  if (parentVarDecl && isIdentifier2(parentVarDecl.id)) {
    vineCompFnCtx.slotsAlias = parentVarDecl.id.name;
  }
};
var analyzeVineModel = ({ vineCompFnCtx }, fnItselfNode) => {
  const vineModelMacroCalls = [];
  _breakableTraverse(fnItselfNode, (node, parent) => {
    if (isVineModel(node)) {
      const foundVarDeclAncestor = parent.find((ancestor) => isVariableDeclarator2(ancestor.node));
      vineModelMacroCalls.push({
        macroCall: node,
        parentVarDecl: foundVarDeclAncestor?.node
      });
    }
  });
  for (const { macroCall, parentVarDecl } of vineModelMacroCalls) {
    if (!parentVarDecl || !isIdentifier2(parentVarDecl.id)) {
      continue;
    }
    const varName = parentVarDecl.id.name;
    if (!macroCall.arguments.length) {
      vineCompFnCtx.vineModels.modelValue = {
        varName,
        modelModifiersName: DEFAULT_MODEL_MODIFIERS_NAME,
        modelOptions: null
      };
      continue;
    } else if (macroCall.arguments.length === 1) {
      if (isStringLiteral2(macroCall.arguments[0])) {
        const modelName = macroCall.arguments[0].value;
        vineCompFnCtx.vineModels[modelName] = {
          varName,
          modelModifiersName: `${modelName}Modifiers`,
          modelOptions: null
        };
      } else {
        vineCompFnCtx.vineModels.modelValue = {
          varName,
          modelModifiersName: DEFAULT_MODEL_MODIFIERS_NAME,
          modelOptions: macroCall.arguments[0]
        };
      }
    } else {
      const modelName = macroCall.arguments[0].value;
      vineCompFnCtx.vineModels[modelName] = {
        varName,
        modelModifiersName: `${modelName}Modifiers`,
        modelOptions: macroCall.arguments[1]
      };
    }
  }
  for (const [modelName, modelDef] of Object.entries(vineCompFnCtx.vineModels)) {
    vineCompFnCtx.bindings[modelName] = VineBindingTypes.PROPS;
    vineCompFnCtx.bindings[modelDef.varName] = VineBindingTypes.SETUP_REF;
  }
  if (vineCompFnCtx.emitsAlias && vineCompFnCtx.emits.length) {
    vineCompFnCtx.bindings[vineCompFnCtx.emitsAlias] = VineBindingTypes.SETUP_CONST;
  }
};
var analyzeRunners = [
  analyzeVineProps,
  analyzeVineEmits,
  analyzeVineExpose,
  analyzeVineSlots,
  analyzeVineModel,
  analyzeVineOptions,
  analyzeVineBindings,
  analyzeVineStyle,
  analyzeVineCustomElement
];
function analyzeDifferentKindVineFunctionDecls(analyzeCtx) {
  const { vineCompFnCtx } = analyzeCtx;
  const { fnItselfNode } = vineCompFnCtx;
  if (!fnItselfNode) {
    return;
  }
  analyzeRunners.forEach((exec) => exec(analyzeCtx, fnItselfNode));
}
function analyzeFileImportStmts(vineFileCtx) {
  const { root } = vineFileCtx;
  const fileImportStmts = getImportStatements(root);
  if (!fileImportStmts.length) {
    return;
  }
  for (const importStmt of fileImportStmts) {
    const source = importStmt.source.value;
    const isImportTypeStmt = importStmt.importKind === "type";
    const allSpecifiers = importStmt.specifiers;
    for (const spec of allSpecifiers) {
      const importMeta = {
        source,
        isType: isImportTypeStmt
      };
      if (isImportSpecifier(spec)) {
        const importedName = isStringLiteral2(spec.imported) ? spec.imported.value : spec.imported.name;
        const localName = spec.local.name;
        if (spec.importKind === "type") {
          importMeta.isType = true;
        } else if (importedName === "default") {
          importMeta.isDefault = true;
        }
        if (source === "vue") {
          vineFileCtx.vueImportAliases[importedName] = localName || importedName;
        }
        vineFileCtx.userImports[localName] = importMeta;
      } else if (isImportNamespaceSpecifier(spec)) {
        importMeta.isNamespace = true;
        vineFileCtx.userImports[spec.local.name] = importMeta;
      } else if (isImportDefaultSpecifier(spec)) {
        importMeta.isDefault = true;
        vineFileCtx.userImports[spec.local.name] = importMeta;
      }
      const specLocalName = spec.local.name;
      importMeta.isUsedInTemplate = (compFnCtx) => isImportUsed(compFnCtx, specLocalName);
    }
  }
  const lastImportStmt = fileImportStmts[fileImportStmts.length - 1];
  vineFileCtx.importsLastLine = lastImportStmt.loc;
}
function createLinkedCodeTag(side, itemLength) {
  return `/* __LINKED_CODE_${side.toUpperCase()}__#${itemLength} */`;
}
function buildVineCompFnCtx(vineCompilerHooks, vineFileCtx, vineFnInfo) {
  const { fnDeclNode, fnName, fnItselfNode } = vineFnInfo;
  const {
    templateReturn,
    templateStringNode
  } = findVineTagTemplateStringReturn(
    fnDeclNode
  );
  const scopeId = hashId(`${vineFileCtx.fileId}:${fnName}`);
  const templateStringQuasiNode = templateStringNode?.quasi.quasis[0];
  const templateSource = templateStringQuasiNode?.value.raw ?? "";
  const vineCompFnCtx = {
    isExportDefault: isExportDefaultDeclaration2(fnDeclNode),
    isAsync: fnItselfNode?.async ?? false,
    isCustomElement: false,
    fnName,
    scopeId,
    fnDeclNode,
    fnItselfNode,
    templateStringNode,
    templateReturn,
    templateSource,
    linkedMacroCalls: [],
    propsDefinitionBy: "annotaion",
    propsAlias: "props",
    emitsAlias: "emits",
    props: {},
    emits: [],
    slots: {},
    slotsAlias: "slots",
    slotsNamesInTemplate: ["default"],
    // Vue component's default slot name
    vineModels: {},
    bindings: {},
    cssBindings: {},
    externalStyleFilePaths: [],
    hoistSetupStmts: [],
    getPropsTypeRecordStr({
      joinStr = ", ",
      isNeedLinkedCodeTag = false
    } = {}) {
      const fields = Object.entries(this.props).map(
        ([propName, propMeta]) => `${(isNeedLinkedCodeTag ? `${createLinkedCodeTag("left", propName.length)}${propName}` : propName) + (propMeta.isRequired ? "" : "?")}: ${propMeta.typeAnnotationRaw}`
      ).filter(Boolean).join(joinStr);
      return `{${fields.length > 0 ? `
${fields}
` : ""}}`;
    }
  };
  const analyzeCtx = {
    vineCompilerHooks,
    vineFileCtx,
    vineCompFnCtx
  };
  analyzeDifferentKindVineFunctionDecls(analyzeCtx);
  return vineCompFnCtx;
}
function analyzeVine(vineCompilerHooks, vineFileCtx, vineCompFnDecls) {
  analyzeFileImportStmts(vineFileCtx);
  vineCompFnDecls.forEach(
    (vineFnCompDecl) => {
      const pickedInfos = getFunctionPickedInfos(vineFnCompDecl);
      pickedInfos.forEach((vineFnInfo) => {
        vineFileCtx.vineCompFns.push(
          buildVineCompFnCtx(
            vineCompilerHooks,
            vineFileCtx,
            vineFnInfo
          )
        );
      });
    }
  );
  const makeErrorOnRefHoistedIdentifiers = (vineFnComp, identifiers) => {
    for (const id of identifiers) {
      const binding = vineFnComp.bindings[id.name];
      if (binding && binding !== VineBindingTypes.LITERAL_CONST) {
        vineCompilerHooks.onError(
          vineWarn({ vineFileCtx }, {
            msg: `Cannot reference "${id.name}" locally declared variables because it will be hoisted outside of component's setup() function.`,
            location: id.loc
          })
        );
      }
    }
  };
  for (const vineFnComp of vineFileCtx.vineCompFns) {
    const allValidatorFnBodys = Object.entries(vineFnComp.props).filter(([_, propMeta]) => Boolean(propMeta.validator)).map(([_, propMeta]) => propMeta.validator?.body);
    for (const validatorFnBody of allValidatorFnBodys) {
      if (!validatorFnBody) {
        continue;
      }
      const identifiers = [];
      walkIdentifiers2(validatorFnBody, (id) => identifiers.push(id));
      makeErrorOnRefHoistedIdentifiers(vineFnComp, identifiers);
    }
    const vineOptionsArg = vineFnComp.options;
    if (vineOptionsArg) {
      const identifiers = [];
      walkIdentifiers2(vineOptionsArg, (id) => identifiers.push(id));
      makeErrorOnRefHoistedIdentifiers(vineFnComp, identifiers);
    }
    if (vineFnComp.vineModels) {
      for (const { modelOptions } of Object.values(vineFnComp.vineModels)) {
        if (!modelOptions) {
          continue;
        }
        const identifiers = [];
        walkIdentifiers2(modelOptions, (id) => identifiers.push(id));
        makeErrorOnRefHoistedIdentifiers(vineFnComp, identifiers);
      }
    }
  }
}

// src/babel-helpers/parse.ts
import { parse } from "@babel/parser";
function babelParse(code, options = {}) {
  const finalOptions = {
    sourceType: "module",
    plugins: ["typescript"],
    errorRecovery: true,
    ...options
  };
  try {
    return parse(code, finalOptions);
  } catch {
    return parse("", finalOptions);
  }
}

// src/transform.ts
import {
  isAssignmentExpression,
  isAwaitExpression,
  isBlockStatement as isBlockStatement2,
  isExpressionStatement,
  isReturnStatement as isReturnStatement2,
  isVariableDeclaration as isVariableDeclaration3,
  traverse as traverse4
} from "@babel/types";

// src/template/traverse.ts
function traverseNode(node, filter, callback) {
  const children = node.children;
  if (!filter(node)) {
    return;
  }
  callback(node);
  if (Array.isArray(children)) {
    for (const childNode of children) {
      traverseNode(childNode, filter, callback);
    }
  }
}
function traverseTemplate(templateAstRootNode, filter, callback) {
  for (const childNode of templateAstRootNode.children) {
    traverseNode(childNode, filter, callback);
  }
}

// src/template/type-predicate.ts
function isTemplateElementNode(node) {
  return node.type === 1;
}
function isComponentNode(node) {
  return node.type === 1 && node.tagType === 1;
}

// src/style/create-import-statement.ts
function createStyleImportStmt(vineFileCtx, vineCompFnCtx, styleMeta, index) {
  if (styleMeta.isExternalFilePathSource) {
    if (!styleMeta.scoped) {
      return `import ${showIf(
        // handle web component styles
        Boolean(vineCompFnCtx.isCustomElement),
        `__${vineCompFnCtx.fnName.toLowerCase()}_styles from `
      )} '${styleMeta.source}';`;
    }
    const styleFileExt = styleMeta.source.slice(
      styleMeta.source.lastIndexOf(".")
    );
    return `import ${showIf(
      // handle web component styles
      Boolean(vineCompFnCtx.isCustomElement),
      `__${vineCompFnCtx.fnName.toLowerCase()}_styles from `
    )}'${styleMeta.source}?vineFileId=${encodeURIComponent(vineFileCtx.fileId.replace(/\.vine\.ts$/, ""))}&type=vine-style-external&scopeId=${vineCompFnCtx.scopeId}&comp=${vineCompFnCtx.fnName}&lang=${styleMeta.lang}&index=${index}${showIf(
      Boolean(styleMeta.scoped),
      "&scoped=true"
    )}&virtual${styleFileExt}';`;
  }
  const styleLangExt = getStyleLangFileExt(styleMeta.lang);
  return `import ${showIf(
    // handle web component styles
    Boolean(vineCompFnCtx.isCustomElement),
    `__${vineCompFnCtx.fnName.toLowerCase()}_styles from `
  )}'${vineFileCtx.fileId.replace(/\.vine\.ts$/, "")}?type=vine-style&scopeId=${vineCompFnCtx.scopeId}&comp=${vineCompFnCtx.fnName}&lang=${styleMeta.lang}${showIf(
    Boolean(styleMeta.scoped),
    "&scoped=true"
  )}&index=${index}&virtual.${styleLangExt}';`;
}

// src/style/order.ts
function topoSort(relationsMap) {
  const visited = {};
  const sorted = [];
  function dfs(node, stack) {
    if (stack[node]) {
      return null;
    }
    if (visited[node])
      return;
    visited[node] = true;
    stack[node] = true;
    for (const depNode of relationsMap[node]) {
      const result = dfs(depNode, stack);
      if (result === null) {
        return null;
      }
    }
    stack[node] = false;
    sorted.push(node);
  }
  for (const node of Object.keys(relationsMap)) {
    dfs(node, {});
  }
  if (sorted.length !== Object.keys(visited).length) {
    return null;
  }
  return sorted;
}
function sortStyleImport(vineFileCtx) {
  const { vineCompFns, styleDefine } = vineFileCtx;
  const relationsMap = Object.fromEntries(
    vineCompFns.map(
      (compFnCtx) => [compFnCtx.fnName, /* @__PURE__ */ new Set()]
    )
  );
  for (const compFnCtx of vineCompFns) {
    const templateAstRootNode = compFnCtx.templateAst;
    if (!templateAstRootNode) {
      continue;
    }
    traverseTemplate(templateAstRootNode, (templateAstNode) => isTemplateElementNode(templateAstNode) && isComponentNode(templateAstNode), (templateAstNode) => {
      relationsMap[compFnCtx.fnName].add(
        templateAstNode.tag
      );
    });
  }
  const sortedStyleImportStmts = (topoSort(relationsMap)?.map(
    (compName) => vineCompFns.find(
      (compFn) => compFn.fnName === compName
    )
  ) ?? []).filter(
    (fnCompCtx) => Boolean(styleDefine[fnCompCtx.scopeId])
  ).map(
    (fnCompCtx) => [fnCompCtx, styleDefine[fnCompCtx.scopeId]]
  ).map(
    ([fnCompCtx, styleMetas]) => styleMetas.map(
      (styleMeta, i) => createStyleImportStmt(
        vineFileCtx,
        fnCompCtx,
        styleMeta,
        i
      )
    )
  ).flat();
  return sortedStyleImportStmts;
}

// src/style/transform-css-var.ts
import { traverse as traverse3 } from "@babel/types";
import MagicString from "magic-string";
function findIdentifierFromExp(scriptContent) {
  const identifiers = [];
  const ast = babelParse(scriptContent);
  traverse3(ast, (node) => {
    if (node.type === "Identifier") {
      identifiers.push(node);
    }
  });
  return identifiers;
}
function genCSSVarsList(cssBindings, propsAlias, bindings, inline = false) {
  let res = "";
  if (cssBindings) {
    for (const cssBindScriptContent in cssBindings) {
      const cssBindValue = cssBindings[cssBindScriptContent];
      const ms = new MagicString(cssBindScriptContent);
      const cssBindKeySgNodes = findIdentifierFromExp(cssBindScriptContent);
      cssBindKeySgNodes.forEach((node) => {
        ms.overwrite(
          node.start,
          node.end,
          // non-inline mode only needs to rewrite the variable to `_ctx.x`
          inline ? genCSSVarsValue(ms, node, bindings, propsAlias) : `_ctx.${cssBindScriptContent.slice(
            node.start,
            node.end
          )}`
        );
      });
      res = `${res}  '${cssBindValue}': (${ms.toString()}),
`;
    }
  }
  return res;
}
function genCSSVarsValue(ms, node, bindings, propsAlias) {
  let res = "";
  const nodeContent = ms.original.slice(
    node.start,
    node.end
  );
  for (const bindingsKey in bindings) {
    const bindingValue = bindings[bindingsKey];
    if (nodeContent === bindingsKey) {
      switch (bindingValue) {
        case VineBindingTypes.PROPS:
        case VineBindingTypes.PROPS_ALIASED:
          res = `${propsAlias}.${nodeContent}`;
          break;
        case VineBindingTypes.SETUP_CONST:
        case VineBindingTypes.SETUP_REACTIVE_CONST:
        case VineBindingTypes.LITERAL_CONST:
          res = nodeContent;
          break;
        case VineBindingTypes.SETUP_MAYBE_REF:
        case VineBindingTypes.SETUP_LET:
          res = `_unref(${nodeContent})`;
          break;
        // The `vineProp` variable is inconsistent with vue here, and vue is `PROPS`
        // Because vine compilation will use `toRefs` processing
        case VineBindingTypes.SETUP_REF:
          res = `${nodeContent}.value`;
          break;
        default:
          res = `_ctx.${nodeContent}`;
      }
    }
  }
  return res;
}
function compileCSSVars(vineCompFnCtx, inline = false) {
  const {
    cssBindings,
    propsAlias,
    bindings
  } = vineCompFnCtx;
  if (!cssBindings)
    return "";
  const varList = genCSSVarsList(
    cssBindings,
    propsAlias,
    bindings,
    inline
  );
  return `_${CSS_VARS_HELPER}(_ctx => ({
${varList}
}))`;
}

// src/template/compose.ts
import { isExportNamedDeclaration as isExportNamedDeclaration2, isFunctionDeclaration as isFunctionDeclaration3, isIdentifier as isIdentifier3, isImportDeclaration as isImportDeclaration2, isImportDefaultSpecifier as isImportDefaultSpecifier2, isImportSpecifier as isImportSpecifier2 } from "@babel/types";
import { compile, ElementTypes as ElementTypes2, NodeTypes as NodeTypes3, parse as parse2 } from "@vue/compiler-dom";
import { compile as ssrCompile } from "@vue/compiler-ssr";
import lineColumn from "line-column";

// src/template/walk.ts
import { ElementTypes, NodeTypes as NodeTypes2 } from "@vue/compiler-dom";
function isWalkable(item) {
  return item && typeof item === "object" && "type" in item && "children" in item && Array.isArray(item.children);
}
function walkVueTemplateAst(root, walkActions) {
  if (!root) {
    return;
  }
  const parents = [];
  let isBreakedWalk = false;
  const breakWalk = () => {
    isBreakedWalk = true;
  };
  const walk = (node) => {
    if (!isWalkable(node) || isBreakedWalk) {
      return;
    }
    parents.push(node);
    walkActions.enter?.(node, parents, breakWalk);
    node.children.forEach((child) => walk(child));
    walkActions.leave?.(node, parents, breakWalk);
    parents.pop();
  };
  root.children.forEach((child) => walk(child));
}
function isElementNode(node) {
  return node.type === NodeTypes2.ELEMENT;
}
function isComponentNode2(node) {
  return isElementNode(node) && node.tagType === ElementTypes.COMPONENT;
}

// src/template/compose.ts
function compileVineTemplate(source, params, { ssr, getParsedAst = false }) {
  const _compile = ssr ? ssrCompile : compile;
  try {
    return {
      ..._compile(source, {
        mode: "module",
        hoistStatic: true,
        cacheHandlers: true,
        prefixIdentifiers: true,
        inline: true,
        ...params
      }),
      templateParsedAst: getParsedAst ? parse2(source, {
        parseMode: "base",
        prefixIdentifiers: true,
        expressionPlugins: [
          "typescript"
        ]
      }) : void 0
    };
  } catch {
    return null;
  }
}
function saveImportSpecifier(mergedImportsMap, importStmts) {
  for (const importStmt of importStmts) {
    const allImportSpecs = importStmt.specifiers;
    const source = importStmt.source.value;
    for (const spec of allImportSpecs) {
      if (isImportSpecifier2(spec)) {
        const specName = isIdentifier3(spec.imported) ? spec.imported.name : spec.imported.value;
        const specAlias = isIdentifier3(spec.local) ? spec.local.name : specName;
        const specMeta = mergedImportsMap.get(source);
        if (!specMeta) {
          mergedImportsMap.set(
            source,
            {
              type: "namedSpecifier",
              specs: /* @__PURE__ */ new Map([[specName, specAlias]])
            }
          );
        } else {
          specMeta.specs.set(specName, specAlias);
        }
      } else if (isImportDefaultSpecifier2(spec)) {
        mergedImportsMap.set(
          source,
          {
            type: "defaultSpecifier",
            localName: spec.local.name
          }
        );
      } else {
        mergedImportsMap.set(
          source,
          {
            type: "namespaceSpecifier",
            localName: spec.local.name
          }
        );
      }
    }
  }
}
function isExportRenderFnNode(node, ssr = false) {
  if (!isExportNamedDeclaration2(node)) {
    return false;
  }
  return isFunctionDeclaration3(node.declaration) && node.declaration.id?.name === (ssr ? "ssrRender" : "render");
}
function getQuasisNode(vineFnCompCtx) {
  return vineFnCompCtx.templateStringNode?.quasi.quasis[0];
}
function computeTemplateErrLocation(vineFileCtx, vineFnCompCtx, errLocation) {
  const quasisNode = getQuasisNode(vineFnCompCtx);
  if (!quasisNode) {
    return;
  }
  const mapper = lineColumn(
    vineFileCtx.fileMagicCode.original,
    { origin: 1 }
    // Babel line/column is 1-based
  );
  const templateStart = quasisNode.start;
  const startOffset = templateStart + errLocation.start.offset;
  const endOffset = templateStart + errLocation.end.offset;
  const start = mapper.fromIndex(startOffset);
  const end = mapper.fromIndex(endOffset);
  if (!start || !end) {
    return;
  }
  const [startColumn, endColumn] = start.line === end.line && start.col === end.col ? [1, Number.POSITIVE_INFINITY] : [start.col, end.col];
  const loc = {
    identifierName: errLocation.source,
    filename: vineFileCtx.fileId,
    start: {
      line: start.line,
      column: startColumn,
      index: startOffset
    },
    end: {
      line: end.line,
      column: endColumn,
      index: endOffset
    }
  };
  return loc;
}
function setVineTemplateAst(vineCompFnCtx, compileResult) {
  const { ast, templateParsedAst } = compileResult;
  vineCompFnCtx.templateAst = ast;
  vineCompFnCtx.templateParsedAst = templateParsedAst;
  if (!ast) {
    return;
  }
  walkVueTemplateAst(ast, {
    enter(node) {
      if (node.type === NodeTypes3.ELEMENT && node.tagType === ElementTypes2.SLOT) {
        const slotNameAttr = node.props.find(
          (prop) => prop.type === NodeTypes3.ATTRIBUTE && prop.name === "name"
        );
        if (slotNameAttr?.value) {
          vineCompFnCtx.slotsNamesInTemplate.push(slotNameAttr.value.content);
        }
      }
    }
  });
}
function createSeparatedTemplateComposer(compilerHooks, ssr) {
  const templateCompileResults = /* @__PURE__ */ new WeakMap();
  const generatedPreambleStmts = /* @__PURE__ */ new WeakMap();
  return {
    templateCompileResults,
    generatedPreambleStmts,
    compileSetupFnReturns: ({
      vineFileCtx,
      vineCompFnCtx,
      templateSource,
      mergedImportsMap,
      bindingMetadata
    }) => {
      let hasTemplateCompileErr = false;
      const compileResult = compileVineTemplate(
        templateSource,
        {
          scopeId: `data-v-${vineCompFnCtx.scopeId}`,
          inline: false,
          bindingMetadata,
          ...compilerHooks.getCompilerCtx()?.options?.vueCompilerOptions ?? {},
          onError: (e) => {
            hasTemplateCompileErr = true;
            compilerHooks.onError(
              vineErr(
                { vineFileCtx, vineCompFnCtx },
                {
                  msg: `[Vine template compile error] ${e.message}`,
                  location: e.loc && computeTemplateErrLocation(vineFileCtx, vineCompFnCtx, e.loc),
                  rawVueTemplateLocation: e.loc
                }
              )
            );
          },
          onWarn: (e) => {
            compilerHooks.onWarn(
              vineWarn(
                { vineFileCtx, vineCompFnCtx },
                {
                  msg: `[Vine template compile warning] ${e.message}`,
                  location: e.loc && computeTemplateErrLocation(vineFileCtx, vineCompFnCtx, e.loc),
                  rawVueTemplateLocation: e.loc
                }
              )
            );
          }
        },
        {
          ssr,
          // Separate mode needs template's original AST,
          // avoid expressions in template to be compiled as '$setup.foo()'
          getParsedAst: true
        }
      );
      if (!compileResult) {
        return "";
      }
      setVineTemplateAst(vineCompFnCtx, compileResult);
      if (hasTemplateCompileErr) {
        return "";
      }
      const { code } = compileResult;
      const generatedCodeAst = babelParse(code);
      saveImportSpecifier(
        mergedImportsMap,
        generatedCodeAst.program.body.filter(
          (stmt) => isImportDeclaration2(stmt)
        )
      );
      let exportRenderFnNode;
      for (const codeStmt of generatedCodeAst.program.body) {
        if (isImportDeclaration2(codeStmt)) {
          continue;
        } else if (isExportRenderFnNode(codeStmt, ssr)) {
          exportRenderFnNode = codeStmt;
          continue;
        }
        appendToMapArray(
          generatedPreambleStmts,
          vineCompFnCtx,
          code.slice(
            codeStmt.start,
            codeStmt.end
          )
        );
      }
      if (!exportRenderFnNode) {
        compilerHooks.onError(vineErr({ vineFileCtx, vineCompFnCtx }, {
          msg: "[Vine Error] Cannot find export render function on template composing"
        }));
        return "";
      }
      templateCompileResults.set(
        vineCompFnCtx,
        code.slice(
          exportRenderFnNode.start,
          exportRenderFnNode.end
        ).replace(
          ssr ? "export function ssrRender" : "export function render",
          ssr ? "function __sfc_ssr_render" : "function __sfc_render"
        )
      );
      const allBindings = { ...bindingMetadata };
      for (const key in vineFileCtx.userImports) {
        const isType = vineFileCtx.userImports[key].isType;
        const isUsedInTemplate = vineFileCtx.userImports[key].isUsedInTemplate?.(vineCompFnCtx);
        if (isUsedInTemplate) {
          if (!isType)
            allBindings[key] = true;
        } else {
          allBindings[key] = false;
        }
      }
      let setupFnReturns = "{ ";
      for (const key in allBindings) {
        if (allBindings[key] === false) {
        } else if (allBindings[key] === true && vineFileCtx.userImports[key].source !== "vue" && !vineFileCtx.userImports[key].source.endsWith(".vue")) {
          setupFnReturns += `get ${key}() { return ${key} }, `;
        } else if (bindingMetadata[key] === VineBindingTypes.SETUP_LET) {
          const setArg = key === "v" ? "_v" : "v";
          setupFnReturns += `get ${key}() { return ${key} }, set ${key}(${setArg}) { ${key} = ${setArg} }, `;
        } else if (bindingMetadata[key] === VineBindingTypes.PROPS) {
        } else {
          setupFnReturns += `${key}, `;
        }
      }
      setupFnReturns = `${setupFnReturns.replace(/, $/, "")} }`;
      return setupFnReturns;
    }
  };
}
function createInlineTemplateComposer(compilerHooks, ssr) {
  const templateCompileResults = /* @__PURE__ */ new WeakMap();
  const generatedPreambleStmts = /* @__PURE__ */ new WeakMap();
  return {
    templateCompileResults,
    generatedPreambleStmts,
    compileSetupFnReturns: ({
      vineFileCtx,
      vineCompFnCtx,
      templateSource,
      mergedImportsMap,
      bindingMetadata
    }) => {
      let hasTemplateCompileErr = false;
      const compileResult = compileVineTemplate(
        templateSource,
        {
          scopeId: `data-v-${vineCompFnCtx.scopeId}`,
          bindingMetadata,
          ...compilerHooks.getCompilerCtx()?.options?.vueCompilerOptions ?? {},
          onError: (e) => {
            if (hasTemplateCompileErr) {
              return;
            }
            hasTemplateCompileErr = true;
            compilerHooks.onError(
              vineErr(
                { vineFileCtx, vineCompFnCtx },
                {
                  msg: `[Vine template compile error] ${e.message}`,
                  location: e.loc && computeTemplateErrLocation(vineFileCtx, vineCompFnCtx, e.loc),
                  rawVueTemplateLocation: e.loc
                }
              )
            );
          },
          onWarn: (e) => {
            compilerHooks.onWarn(
              vineWarn(
                { vineFileCtx, vineCompFnCtx },
                {
                  msg: `[Vine template compile warning] ${e.message}`,
                  location: e.loc && computeTemplateErrLocation(vineFileCtx, vineCompFnCtx, e.loc),
                  rawVueTemplateLocation: e.loc
                }
              )
            );
          }
        },
        { ssr }
      );
      if (!compileResult) {
        return "";
      }
      const { preamble, code } = compileResult;
      setVineTemplateAst(vineCompFnCtx, compileResult);
      if (hasTemplateCompileErr) {
        return "";
      }
      const preambleAst = babelParse(preamble);
      saveImportSpecifier(
        mergedImportsMap,
        preambleAst.program.body.filter(
          (stmt) => isImportDeclaration2(stmt)
        )
      );
      const preambleStmts = preambleAst.program.body.filter(
        (stmt) => !isImportDeclaration2(stmt)
      );
      preambleStmts.forEach((stmt) => appendToMapArray(
        generatedPreambleStmts,
        vineCompFnCtx,
        preamble.slice(
          stmt.start,
          stmt.end
        )
      ));
      templateCompileResults.set(vineCompFnCtx, code);
      return code;
    }
  };
}

// src/transform.ts
function wrapWithAsyncContext(isNeedResult, exprSourceCode) {
  return isNeedResult ? `(
    ([__temp,__restore] = _withAsyncContext(() => ${exprSourceCode})),
    __temp = await __temp,
    __restore(),
    __temp
    );` : `;(
    ([__temp,__restore] = _withAsyncContext(() => ${exprSourceCode})),
    await __temp,
    __restore()
  );`;
}
function mayContainAwaitExpr(vineFnBodyStmt) {
  let awaitExpr;
  const isVarDecl = isVariableDeclaration3(vineFnBodyStmt);
  const isAssignExpr = isAssignmentExpression(vineFnBodyStmt);
  const isExprStmt = isExpressionStatement(vineFnBodyStmt);
  if (!(isVarDecl || isAssignExpr || isExprStmt)) {
    return null;
  }
  if (isExprStmt && vineFnBodyStmt.expression.type !== "AwaitExpression") {
    return null;
  }
  const isNeedResult = isVarDecl || isAssignExpr;
  try {
    traverse4(vineFnBodyStmt, (descendant) => {
      if (isAwaitExpression(descendant)) {
        awaitExpr = descendant;
        throw new Error(EXPECTED_ERROR);
      }
    });
  } catch (error) {
    if (error.message === EXPECTED_ERROR) {
      return {
        isNeedResult,
        awaitExpr
      };
    }
    throw error;
  }
  return null;
}
function registerImport(mergedImportsMap, importSource, specifier, alias) {
  const vueVineImports = mergedImportsMap.get(importSource);
  if (!vueVineImports) {
    mergedImportsMap.set(importSource, {
      type: "namedSpecifier",
      specs: /* @__PURE__ */ new Map([[specifier, alias]])
    });
  } else {
    const vueImportsSpecs = vueVineImports.specs;
    vueImportsSpecs.set(specifier, alias);
  }
}
function setupCodeGenerationForVineModel(vineFnCompCtx) {
  let modelCodeGen = [];
  for (const [modelName, modelDef] of Object.entries(vineFnCompCtx.vineModels)) {
    const { varName } = modelDef;
    modelCodeGen.push(
      `const ${varName} = _${USE_MODEL_HELPER}(__props, '${modelName}')`
    );
  }
  return `
${modelCodeGen.join("\n")}
`;
}
function propsOptionsCodeGeneration(vineFileCtx, vineCompFnCtx) {
  const segementsFromProps = Object.entries(vineCompFnCtx.props).map(([propName, propMeta]) => {
    const metaFields = [];
    if (propMeta.isRequired) {
      metaFields.push("required: true");
    }
    if (propMeta.isBool) {
      metaFields.push("type: Boolean");
    }
    if (propMeta.validator) {
      metaFields.push(`validator: ${vineFileCtx.originCode.slice(
        propMeta.validator.start,
        propMeta.validator.end
      )}`);
    }
    return `${propName}: { ${showIf(
      metaFields.filter(Boolean).length > 0,
      filterJoin(metaFields, ", "),
      "/* Simple prop */"
    )} },`;
  });
  const segmentsFromVineModel = Object.entries(vineCompFnCtx.vineModels).reduce((segments, [modelName, modelDef]) => {
    const { modelModifiersName, modelOptions } = modelDef;
    segments.push(
      `${modelName}: ${modelOptions ? vineFileCtx.originCode.slice(
        modelOptions.start,
        modelOptions.end
      ) : "{}"},`
    );
    segments.push(
      `${modelModifiersName}: {},`
    );
    return segments;
  }, []);
  return [
    ...segementsFromProps,
    ...segmentsFromVineModel
  ];
}
function transformFile(vineFileCtx, compilerHooks, inline = true, ssr = false) {
  const isDev = compilerHooks.getCompilerCtx().options.envMode !== "production";
  const ms = vineFileCtx.fileMagicCode;
  const styleImportStmts = sortStyleImport(vineFileCtx);
  const mergedImportsMap = /* @__PURE__ */ new Map();
  let isPrependedUseDefaults = false;
  const {
    templateCompileResults,
    generatedPreambleStmts,
    compileSetupFnReturns
  } = inline ? createInlineTemplateComposer(compilerHooks, ssr) : createSeparatedTemplateComposer(compilerHooks, ssr);
  for (const vineCompFnCtx of vineFileCtx.vineCompFns) {
    const setupFnReturns = compileSetupFnReturns({
      vineFileCtx,
      vineCompFnCtx,
      templateSource: vineCompFnCtx.templateSource,
      mergedImportsMap,
      bindingMetadata: vineCompFnCtx.bindings
    });
    const isNeedUseDefaults = Object.values(vineCompFnCtx.props).some((meta) => Boolean(meta.default));
    let vueImportsMeta = mergedImportsMap.get("vue");
    if (!vueImportsMeta) {
      vueImportsMeta = {
        type: "namedSpecifier",
        specs: /* @__PURE__ */ new Map()
      };
      mergedImportsMap.set("vue", vueImportsMeta);
    }
    const vueImportsSpecs = vueImportsMeta.specs;
    if (!vueImportsSpecs.has(DEFINE_COMPONENT_HELPER)) {
      vueImportsSpecs.set(DEFINE_COMPONENT_HELPER, `_${DEFINE_COMPONENT_HELPER}`);
    }
    if (!vueImportsSpecs.has(CSS_VARS_HELPER) && vineCompFnCtx.cssBindings) {
      vueImportsSpecs.set(CSS_VARS_HELPER, `_${CSS_VARS_HELPER}`);
      if (inline) {
        vueImportsSpecs.set(UN_REF_HELPER, `_${UN_REF_HELPER}`);
      }
    }
    if (Object.entries(vineCompFnCtx.slots).length > 0) {
      vueImportsSpecs.set(USE_SLOT_HELPER, `_${USE_SLOT_HELPER}`);
    }
    const vineCompFnStart = vineCompFnCtx.fnDeclNode.start;
    const vineCompFnEnd = vineCompFnCtx.fnDeclNode.end;
    const vineCompFnBody = vineCompFnCtx.fnItselfNode?.body;
    if (isBlockStatement2(vineCompFnBody)) {
      let hasAwait = false;
      for (const vineFnBodyStmt of vineCompFnBody.body) {
        const mayContain = mayContainAwaitExpr(vineFnBodyStmt);
        if (!mayContain || !mayContain.awaitExpr) {
          continue;
        }
        vueImportsSpecs.set(WITH_ASYNC_CONTEXT_HELPER, `_${WITH_ASYNC_CONTEXT_HELPER}`);
        const { awaitExpr, isNeedResult } = mayContain;
        hasAwait = true;
        ms.update(
          awaitExpr.start,
          awaitExpr.end,
          wrapWithAsyncContext(
            isNeedResult,
            ms.original.slice(
              awaitExpr.argument.start,
              awaitExpr.argument.end
            )
          )
        );
      }
      const firstStmt = vineCompFnBody.body[0];
      const lastStmt = vineCompFnBody.body[vineCompFnBody.body.length - 1];
      ms.remove(vineCompFnStart, firstStmt.start);
      ms.remove(lastStmt.end, vineCompFnEnd);
      vineCompFnBody.body.forEach((stmt) => {
        if (isStatementContainsVineMacroCall(stmt) || isReturnStatement2(stmt)) {
          ms.remove(stmt.start, stmt.end);
        }
      });
      const setupCtxDestructFormalParams = [];
      if (vineCompFnCtx.emits.length > 0) {
        setupCtxDestructFormalParams.push({
          field: "emit",
          alias: "__emit"
        });
        ms.prependLeft(
          firstStmt.start,
          `const ${vineCompFnCtx.emitsAlias} = __emit;
`
        );
      }
      setupCtxDestructFormalParams.push({
        field: "expose",
        alias: "__expose"
      });
      let setupFormalParams = `__props${setupCtxDestructFormalParams.length > 0 ? `, { ${setupCtxDestructFormalParams.map(
        ({ field, alias }) => `${field}${showIf(Boolean(alias), `: ${alias}`)}`
      ).join(", ")} }` : " /* No setup ctx destructuring */"}`;
      if (Object.entries(vineCompFnCtx.slots).length > 0) {
        ms.prependLeft(
          firstStmt.start,
          `const ${vineCompFnCtx.slotsAlias} = _${USE_SLOT_HELPER}();
`
        );
      }
      if (Object.entries(vineCompFnCtx.vineModels).length > 0) {
        registerImport(
          mergedImportsMap,
          "vue",
          USE_MODEL_HELPER,
          `_${USE_MODEL_HELPER}`
        );
        ms.prependLeft(
          firstStmt.start,
          setupCodeGenerationForVineModel(vineCompFnCtx)
        );
      }
      if (Array.from(
        Object.entries(vineCompFnCtx.cssBindings)
      ).length > 0) {
        ms.prependLeft(
          firstStmt.start,
          `
${compileCSSVars(vineCompFnCtx, inline)}
`
        );
      }
      if (Object.values(vineCompFnCtx.props).some((meta) => Boolean(meta.isFromMacroDefine))) {
        registerImport(
          mergedImportsMap,
          "vue",
          TO_REFS_HELPER,
          `_${TO_REFS_HELPER}`
        );
        const propsFromMacro = Object.entries(vineCompFnCtx.props).filter(([_, meta]) => Boolean(meta.isFromMacroDefine)).map(([propName, _]) => propName);
        ms.prependLeft(
          firstStmt.start,
          `const { ${propsFromMacro.join(", ")} } = _toRefs(${vineCompFnCtx.propsAlias});
`
        );
      }
      let propsDeclarationStmt = `const ${vineCompFnCtx.propsAlias} = __props;
`;
      if (isNeedUseDefaults && !isPrependedUseDefaults) {
        isPrependedUseDefaults = true;
        registerImport(
          mergedImportsMap,
          "vue-vine",
          USE_DEFAULTS_HELPER,
          `_${USE_DEFAULTS_HELPER}`
        );
        propsDeclarationStmt = `const ${vineCompFnCtx.propsAlias} = _${USE_DEFAULTS_HELPER}(__props, {
${Object.entries(vineCompFnCtx.props).filter(([_, propMeta]) => Boolean(propMeta.default)).map(([propName, propMeta]) => `  ${propName}: ${ms.original.slice(
          propMeta.default.start,
          propMeta.default.end
        )}`).join(",\n")}
})
`;
      }
      ms.prependLeft(
        firstStmt.start,
        propsDeclarationStmt
      );
      if (hasAwait) {
        ms.prependLeft(
          firstStmt.start,
          "let __temp, __restore;\n"
        );
      }
      if (vineCompFnCtx.expose) {
        ms.appendRight(
          lastStmt.end,
          `
__expose(${ms.original.slice(
            vineCompFnCtx.expose.start,
            vineCompFnCtx.expose.end
          )});
`
        );
      } else {
        ms.prependLeft(
          firstStmt.start,
          "\n__expose();\n"
        );
      }
      ms.appendRight(lastStmt.end, `
return ${setupFnReturns};`);
      ms.prependLeft(firstStmt.start, `${vineCompFnCtx.isAsync ? "async " : ""}setup(${setupFormalParams}) {
`);
      ms.appendRight(lastStmt.end, "\n}");
      const emitsKeys = [
        ...vineCompFnCtx.emits.map((emit) => `'${emit}'`),
        ...Object.keys(vineCompFnCtx.vineModels).map((modelName) => `'update:${modelName}'`)
      ];
      const propsOptionFields = propsOptionsCodeGeneration(
        vineFileCtx,
        vineCompFnCtx
      );
      ms.prependLeft(firstStmt.start, `const __vine = _defineComponent({
${// Some basic component options
      vineCompFnCtx.options ? `...${ms.original.slice(
        vineCompFnCtx.options.start,
        vineCompFnCtx.options.end
      )},` : `name: '${vineCompFnCtx.fnName}',`}
${propsOptionFields.length > 0 ? `props: {
${propsOptionsCodeGeneration(
        vineFileCtx,
        vineCompFnCtx
      ).join("\n")}
},` : "/* No props */"}
${emitsKeys.length > 0 ? `emits: [${emitsKeys.join(", ")}],` : "/* No emits */"}
`);
      ms.appendRight(lastStmt.end, "\n})");
      ms.prependLeft(firstStmt.start, `
export const ${vineCompFnCtx.fnName} = (() => {
${// Prepend all generated preamble statements
      generatedPreambleStmts.get(vineCompFnCtx)?.join("\n") ?? ""}
`);
      if (vineCompFnCtx.isExportDefault) {
        ms.appendRight(
          ms.length(),
          `

export default ${vineCompFnCtx.fnName};

`
        );
      }
      ms.appendRight(lastStmt.end, `
${inline ? "" : `${templateCompileResults.get(vineCompFnCtx) ?? ""}
__vine.${ssr ? "ssrRender" : "render"} = ${ssr ? "__sfc_ssr_render" : "__sfc_render"}`}
${showIf(
        Boolean(vineFileCtx.styleDefine[vineCompFnCtx.scopeId]),
        `__vine.__scopeId = 'data-v-${vineCompFnCtx.scopeId}';`
      )}
${isDev ? `__vine.__hmrId = '${vineCompFnCtx.scopeId}';` : ""}
${showIf(
        // handle Web Component styles
        Boolean(vineCompFnCtx.isCustomElement),
        `__vine.styles = [__${vineCompFnCtx.fnName.toLowerCase()}_styles];
`
      )}
return __vine
})();`);
      if (isDev) {
        ms.appendRight(
          ms.length(),
          `

typeof __VUE_HMR_RUNTIME__ !== "undefined" && __VUE_HMR_RUNTIME__.createRecord(${vineCompFnCtx.fnName}.__hmrId, ${vineCompFnCtx.fnName});
`
        );
      }
    }
  }
  if (isDev) {
    ms.appendRight(
      ms.length(),
      /* js */
      `export const _rerender_only = ${vineFileCtx.renderOnly}
export const _rerender_vcf_fn_name = ${vineFileCtx.hmrCompFnsName ? `"${vineFileCtx.hmrCompFnsName}"` : '""'}
import.meta.hot?.accept((mod) => {
if (!mod){return;}
const { _rerender_only, _rerender_vcf_fn_name } = mod;
if (!_rerender_vcf_fn_name){return;}
const component = mod[_rerender_vcf_fn_name];
if (_rerender_only) {
  __VUE_HMR_RUNTIME__.rerender(component.__hmrId, component.render);
} else {
  __VUE_HMR_RUNTIME__.reload(component.__hmrId, component);
}
});`
    );
  }
  ms.prepend(`
${styleImportStmts.join("\n")}
`);
  for (const [importSource, importMeta] of mergedImportsMap) {
    if (importMeta.type === "namedSpecifier") {
      const { specs } = importMeta;
      const specifiers = [...specs.entries()];
      const importStmt = `import { ${specifiers.map(
        ([specifier, alias]) => `${specifier}${showIf(Boolean(alias), ` as ${alias}`)}`
      ).join(", ")} } from '${importSource}';
`;
      ms.prepend(importStmt);
    } else if (importMeta.type === "defaultSpecifier") {
      const importStmt = `import ${importMeta.localName} from '${importSource}';
`;
      ms.prepend(importStmt);
    } else if (importMeta.type === "namespaceSpecifier") {
      const importStmt = `import * as ${importMeta.localName} from '${importSource}';
`;
      ms.prepend(importStmt);
    }
  }
}

// src/validate.ts
import process from "node:process";
import {
  isArrayExpression as isArrayExpression2,
  isIdentifier as isIdentifier4,
  isObjectExpression,
  isObjectProperty,
  isStringLiteral as isStringLiteral3,
  isTaggedTemplateExpression as isTaggedTemplateExpression3,
  isTemplateLiteral as isTemplateLiteral2,
  isTSFunctionType,
  isTSMethodSignature as isTSMethodSignature2,
  isTSPropertySignature as isTSPropertySignature2,
  isTSTypeAnnotation,
  isTSTypeLiteral as isTSTypeLiteral2,
  isVariableDeclaration as isVariableDeclaration4,
  isVariableDeclarator as isVariableDeclarator3,
  traverse as traverse5
} from "@babel/types";
function wrapVineValidatorWithLog(validators) {
  return process.env.VINE_DEV_VITEST === "true" ? validators.map((validator) => (...args) => {
    const isPass = validator(...args);
    console.log(`${colorful(" VINE VALIDATE ", ["bgGreen", "white"])} ${validator.name} => ${colorful(isPass ? "PASS" : "FAIL", [isPass ? "green" : "red"])}`);
    return isPass;
  }) : validators;
}
function vitestLogMacroAssert(macroName, isPass) {
  if (process.env.VINE_DEV_VITEST !== "true") {
    return;
  }
  console.log(`${colorful(" VINE MACRO ASSERT ", ["bgMagenta", "white"])} ${macroName} => ${colorful(isPass ? "PASS" : "FAIL", [isPass ? "green" : "red"])}`);
}
function validateNoOutsideMacroCalls({ vineCompilerHooks, vineFileCtx, vineCompFns }, root) {
  let isMacroInsideVineCompFn = true;
  traverse5(root, (node) => {
    if (!isVineMacroCallExpression(node)) {
      return;
    }
    isMacroInsideVineCompFn = vineCompFns.some(
      (vineCompFn) => isDescendant(vineCompFn, node)
    );
    if (!isMacroInsideVineCompFn) {
      vineCompilerHooks.onError(
        vineErr(
          { vineFileCtx },
          {
            msg: "Vine macro calls must be inside Vue Vine component function!",
            location: node.loc
          }
        )
      );
    }
  });
  return isMacroInsideVineCompFn;
}
function validateVineTemplateStringUsage({ vineCompilerHooks, vineFileCtx }, vineCompFn) {
  let isVineTemplateUsagePass = true;
  let vineTemplateStrCount = 0;
  traverse5(vineCompFn, (node) => {
    if (!isVineTaggedTemplateString(node)) {
      return;
    }
    vineTemplateStrCount += 1;
    if (vineTemplateStrCount > 1) {
      vineCompilerHooks.onError(
        vineErr(
          { vineFileCtx },
          {
            msg: "Multiple vine tagged template are not allowed inside Vine component function",
            location: node.loc
          }
        )
      );
      isVineTemplateUsagePass = false;
    }
    if (isTagTemplateStringContainsInterpolation(node)) {
      vineCompilerHooks.onError(
        vineErr(
          { vineFileCtx },
          {
            msg: "Vine template string are not allowed to contain interpolation!",
            location: node.loc
          }
        )
      );
      isVineTemplateUsagePass = false;
    }
  });
  return isVineTemplateUsagePass;
}
function assertMacroCallMustBeBare({ vineCompilerHooks, vineFileCtx }, macroCallNode, parent) {
  const isBareCall = !!(parent && parent.length > 0 && parent[parent.length - 1]?.node.type === "ExpressionStatement");
  if (!isBareCall) {
    vineCompilerHooks.onError(
      vineErr(
        { vineFileCtx },
        {
          msg: `\`${getVineMacroCalleeName(macroCallNode)}\` call must be a bare call`,
          location: macroCallNode.loc
        }
      )
    );
  }
  return isBareCall;
}
function assertVineStyleUsage({ vineCompilerHooks, vineFileCtx }, vineStyleMacroCallNode) {
  const vineStyleArgsLength = vineStyleMacroCallNode.arguments.length;
  const macroCallee = vineStyleMacroCallNode.callee;
  if (vineStyleArgsLength !== 1) {
    vineCompilerHooks.onError(
      vineErr(
        { vineFileCtx },
        {
          msg: `\`vineStyle\` ${vineStyleArgsLength > 1 ? "can only" : "must"} have one string argument'`,
          location: macroCallee.loc
        }
      )
    );
    return false;
  }
  const theOnlyArg = vineStyleMacroCallNode.arguments[0];
  if (isStringLiteral3(theOnlyArg)) {
  } else if (isTemplateLiteral2(theOnlyArg)) {
    if (theOnlyArg.expressions.length > 0) {
      vineCompilerHooks.onError(
        vineErr(
          { vineFileCtx },
          {
            msg: "vineStyle argument must be a plain string, template string interpolation is not allowed",
            location: theOnlyArg.loc
          }
        )
      );
      return false;
    }
  } else if (isTaggedTemplateExpression3(theOnlyArg)) {
    if (theOnlyArg.tag && isIdentifier4(theOnlyArg.tag) && !SUPPORTED_CSS_LANGS.includes(theOnlyArg.tag.name)) {
      vineCompilerHooks.onError(
        vineErr(
          { vineFileCtx },
          {
            msg: "vineStyle CSS language only supports: `css`, `scss`, `sass`, `less`, `stylus` and `postcss`",
            location: theOnlyArg.tag.loc
          }
        )
      );
      return false;
    }
  } else {
    vineCompilerHooks.onError(
      vineErr(
        { vineFileCtx },
        {
          msg: "vineStyle's argument can only be string",
          location: theOnlyArg.loc
        }
      )
    );
    return false;
  }
  return true;
}
function assertMacroCanOnlyHaveOneObjLiteralArg({ vineCompilerHooks, vineFileCtx }, macroCallNode) {
  const macroCallArgsLength = macroCallNode.arguments.length;
  const macroCallee = macroCallNode.callee;
  const errMsg = `\`${macroCallee.name}\` ${macroCallArgsLength > 1 ? "can only" : "must"} have one object literal argument`;
  if (macroCallArgsLength === 1) {
    const theOnlyArg = macroCallNode.arguments[0];
    if (!isObjectExpression(theOnlyArg)) {
      vineCompilerHooks.onError(
        vineErr(
          { vineFileCtx },
          {
            msg: errMsg,
            location: theOnlyArg.loc
          }
        )
      );
      return false;
    }
  } else {
    vineCompilerHooks.onError(
      vineErr(
        { vineFileCtx },
        {
          msg: errMsg,
          location: macroCallee.loc
        }
      )
    );
    return false;
  }
  return true;
}
function assertMacroCanOnlyHaveOneTypeParam({ vineCompilerHooks, vineFileCtx }, macroCallNode) {
  const macroCallee = macroCallNode.callee;
  const typeParams = macroCallNode.typeParameters?.params;
  const typeParamsLength = typeParams?.length;
  const errMsg = `\`${macroCallee.name}\` ${typeParamsLength && typeParamsLength > 1 ? "can only" : "must"} have 1 type parameter`;
  if (typeParamsLength !== 1) {
    vineCompilerHooks.onError(
      vineErr(
        { vineFileCtx },
        {
          msg: errMsg,
          location: macroCallNode.callee.loc
        }
      )
    );
    return false;
  }
  return true;
}
function assertMacroVariableDeclarationMustBeIdentifier({ vineCompilerHooks, vineFileCtx }, macroName, macroCallNode, parent, { isMustBeInsideVarDecl } = { isMustBeInsideVarDecl: false }) {
  const varDeclaratorThatMayBeInside = parent?.find((ancestor) => isVariableDeclarator3(ancestor.node));
  if (varDeclaratorThatMayBeInside) {
    const varDeclThatMayBeInsideNode = varDeclaratorThatMayBeInside.node;
    if (!isIdentifier4(varDeclThatMayBeInsideNode.id)) {
      vineCompilerHooks.onError(
        vineErr(
          { vineFileCtx },
          {
            msg: `the declaration of macro \`${macroName}\` call must be an identifier`,
            location: varDeclThatMayBeInsideNode.id.loc
          }
        )
      );
      return false;
    }
  } else if (isMustBeInsideVarDecl) {
    vineCompilerHooks.onError(
      vineErr(
        { vineFileCtx },
        {
          msg: `the declaration of \`${macroName}\` macro call must be inside a variable declaration`,
          location: macroCallNode.loc
        }
      )
    );
    return false;
  }
  return true;
}
function assertVineEmitsUsage(validatorCtx, macroCallNode, parent) {
  const { vineCompilerHooks, vineFileCtx } = validatorCtx;
  let isVineEmitsUsageCorrect = true;
  const callArgs = macroCallNode.arguments;
  const typeParams = macroCallNode.typeParameters?.params;
  if (!typeParams || typeParams.length === 0) {
    const isVineEmitsArgCorrect = !!(callArgs.length === 1 && isArrayExpression2(callArgs[0]) && callArgs[0].elements.every((el) => isStringLiteral3(el)));
    if (!isVineEmitsArgCorrect) {
      vineCompilerHooks.onError(
        vineErr(
          { vineFileCtx },
          {
            msg: "`vineEmits` macro must have a type parameter or an array of string for event names",
            location: callArgs[0]?.loc
          }
        )
      );
    }
    isVineEmitsUsageCorrect = isVineEmitsArgCorrect;
  } else {
    const theOnlyTypeParam = typeParams?.[0];
    if (!isTSTypeLiteral2(theOnlyTypeParam)) {
      vineCompilerHooks.onError(
        vineErr(
          { vineFileCtx },
          {
            msg: "Vue Vine component function's vineEmits type must be object literal!",
            location: theOnlyTypeParam?.loc
          }
        )
      );
      isVineEmitsUsageCorrect = false;
    }
    const properties = theOnlyTypeParam?.members;
    if (!properties.every((prop) => isTSPropertySignature2(prop))) {
      isVineEmitsUsageCorrect = false;
      vineCompilerHooks.onError(
        vineErr(
          { vineFileCtx },
          {
            msg: "Vue Vine component function's vineEmits type must be object literal! And all properties' key must be string literal or identifier",
            location: theOnlyTypeParam?.loc
          }
        )
      );
    }
  }
  if (!assertMacroVariableDeclarationMustBeIdentifier(
    validatorCtx,
    "vineEmits",
    macroCallNode,
    parent
  )) {
    isVineEmitsUsageCorrect = false;
  }
  return isVineEmitsUsageCorrect;
}
function assertSlotMethodSignature(validatorCtx, methodSignature) {
  const { vineCompilerHooks, vineFileCtx } = validatorCtx;
  const params = methodSignature.parameters;
  const errMsg = "Function signature of `vineSlots` definition can only have one parameter named `props`";
  if (params.length !== 1) {
    vineCompilerHooks.onError(
      vineErr(
        { vineFileCtx },
        {
          msg: errMsg,
          location: methodSignature.loc
        }
      )
    );
    return false;
  }
  const theSignatureOnlyParam = params[0];
  let isNameNotProps = false;
  if (!isIdentifier4(theSignatureOnlyParam) || (isNameNotProps = theSignatureOnlyParam.name !== "props")) {
    vineCompilerHooks.onError(
      vineErr(
        { vineFileCtx },
        {
          msg: `${errMsg}${isNameNotProps ? `, and its parameter name must be \`props\`, but got \`${theSignatureOnlyParam.name}\`` : ""}`,
          location: theSignatureOnlyParam.loc
        }
      )
    );
    return false;
  }
  const paramTypeAnnotation = theSignatureOnlyParam.typeAnnotation?.typeAnnotation;
  if (!paramTypeAnnotation || !isTSTypeLiteral2(paramTypeAnnotation)) {
    vineCompilerHooks.onError(
      vineErr(
        { vineFileCtx },
        {
          msg: `${errMsg}, and its type annotation must be object literal`,
          location: theSignatureOnlyParam.loc
        }
      )
    );
    return false;
  }
  return true;
}
function assertSlotPropertySignature(validatorCtx, propertySignature) {
  const { vineCompilerHooks, vineFileCtx } = validatorCtx;
  const typeAnnotation = propertySignature.typeAnnotation?.typeAnnotation;
  if (!typeAnnotation || !isTSFunctionType(typeAnnotation)) {
    vineCompilerHooks.onError(
      vineErr(
        { vineFileCtx },
        {
          msg: "Properties of `vineSlots` can only have function type annotation",
          location: propertySignature.loc
        }
      )
    );
    return false;
  }
  const params = typeAnnotation.parameters;
  const errMsg = "Function signature of `vineSlots` can only have one parameter named `props`";
  const firstParam = params[0];
  if (params.length !== 1 || !isIdentifier4(firstParam)) {
    vineCompilerHooks.onError(
      vineErr(
        { vineFileCtx },
        {
          msg: errMsg,
          location: typeAnnotation.loc
        }
      )
    );
    return false;
  }
  const paramTypeAnnotation = firstParam.typeAnnotation?.typeAnnotation;
  if (!paramTypeAnnotation || !isTSTypeLiteral2(paramTypeAnnotation)) {
    vineCompilerHooks.onError(
      vineErr(
        { vineFileCtx },
        {
          msg: `${errMsg}, and its type annotation must be object literal`,
          location: firstParam.loc
        }
      )
    );
    return false;
  }
  return true;
}
function assertVineSlotsUsage(validatorCtx, macroCallNode, parent) {
  const { vineCompilerHooks, vineFileCtx } = validatorCtx;
  let isVineSlotsUsageCorrect = true;
  if (!assertMacroCanOnlyHaveOneTypeParam(validatorCtx, macroCallNode)) {
    isVineSlotsUsageCorrect = false;
  } else {
    const typeParams = macroCallNode.typeParameters?.params;
    const theOnlyTypeParam = typeParams?.[0];
    if (!isTSTypeLiteral2(theOnlyTypeParam)) {
      vineCompilerHooks.onError(
        vineErr(
          { vineFileCtx },
          {
            msg: "Vue Vine component function's vineSlots type must be object literal!",
            location: theOnlyTypeParam?.loc
          }
        )
      );
      isVineSlotsUsageCorrect = false;
    }
    const slotSignatures = theOnlyTypeParam?.members ?? [];
    if (!slotSignatures.every((prop) => {
      if (isTSMethodSignature2(prop))
        return true;
      if (isTSPropertySignature2(prop) && isTSFunctionType(prop.typeAnnotation?.typeAnnotation)) {
        return true;
      }
      return false;
    })) {
      isVineSlotsUsageCorrect = false;
      vineCompilerHooks.onError(
        vineErr(
          { vineFileCtx },
          {
            msg: "Every property of Vue Vine component function's `vineSlots` type must be function signature",
            location: theOnlyTypeParam?.loc
          }
        )
      );
    }
    slotSignatures.forEach((signature) => {
      if (isTSMethodSignature2(signature) && !assertSlotMethodSignature(validatorCtx, signature)) {
        isVineSlotsUsageCorrect = false;
      } else if (isTSPropertySignature2(signature) && !assertSlotPropertySignature(validatorCtx, signature)) {
        isVineSlotsUsageCorrect = false;
      }
    });
  }
  if (!assertMacroVariableDeclarationMustBeIdentifier(
    validatorCtx,
    "vineSlots",
    macroCallNode,
    parent
  )) {
    isVineSlotsUsageCorrect = false;
  }
  return isVineSlotsUsageCorrect;
}
function assertVineModelDefaultDuplicated(validatorCtx, modelCtx, macroCallNode, extraAssert) {
  const { vineCompilerHooks, vineFileCtx } = validatorCtx;
  const macroCallArgs = macroCallNode.arguments;
  const isAlreadyHasDefault = modelCtx.hasDefaultModel;
  let isThisOneAsDefault = false;
  const isExtraAssertValid = Object.values(extraAssert).every(Boolean);
  if (macroCallArgs?.length === 0 && isExtraAssertValid) {
    isThisOneAsDefault = true;
  } else if (macroCallArgs?.length === 1 && isObjectExpression(macroCallArgs[0]) && isExtraAssertValid) {
    isThisOneAsDefault = true;
  }
  if (isThisOneAsDefault) {
    if (isAlreadyHasDefault) {
      vineCompilerHooks.onError(
        vineErr(
          { vineFileCtx },
          {
            msg: "Vue Vine component function can only have one default model",
            location: macroCallNode.loc
          }
        )
      );
      return false;
    }
    modelCtx.hasDefaultModel = true;
  }
  return true;
}
function assertVineModelUsage(validatorCtx, modelCtx, macroCallNode, parent) {
  const { vineCompilerHooks, vineFileCtx } = validatorCtx;
  let isVineModelUsageCorrect = true;
  const typeParams = macroCallNode.typeParameters?.params;
  const macroCallArgs = macroCallNode.arguments;
  const lastArg = macroCallArgs?.[macroCallArgs.length - 1];
  if (typeParams?.length === 0) {
    if (macroCallArgs?.length === 0) {
    } else if (isObjectExpression(lastArg) && !lastArg.properties.some((prop) => {
      if (isObjectProperty(prop) && isIdentifier4(prop.key) && prop.key.name === "default") {
        return true;
      }
      return false;
    })) {
      vineCompilerHooks.onError(
        vineErr(
          { vineFileCtx },
          {
            msg: "If `vineModel` macro call doesn't have type parameter, it must have a `default` field in the options object literal",
            location: lastArg.loc
          }
        )
      );
      isVineModelUsageCorrect = false;
    }
  }
  let isInsideVarDecl = false;
  if (!assertMacroVariableDeclarationMustBeIdentifier(
    validatorCtx,
    "vineModel",
    macroCallNode,
    parent,
    {
      isMustBeInsideVarDecl: true
    }
  )) {
    isVineModelUsageCorrect = false;
  } else {
    isInsideVarDecl = true;
  }
  if (macroCallArgs?.length > 2) {
    vineCompilerHooks.onError(
      vineErr(
        { vineFileCtx },
        {
          msg: "`vineModel` macro call can only have at most 2 arguments",
          location: macroCallNode.loc
        }
      )
    );
    isVineModelUsageCorrect = false;
  } else {
    isVineModelUsageCorrect = assertVineModelDefaultDuplicated(
      validatorCtx,
      modelCtx,
      macroCallNode,
      {
        isInsideVarDecl
      }
    );
  }
  return isVineModelUsageCorrect;
}
function validateMacrosUsage(validatorCtx, vineCompFn) {
  const { vineCompilerHooks, vineFileCtx } = validatorCtx;
  const macroCountMap = {
    vineStyle: {
      count: 0,
      asserts: [
        assertVineStyleUsage
      ]
    },
    vineEmits: {
      count: 0,
      asserts: [
        assertVineEmitsUsage
      ]
    },
    vineSlots: {
      count: 0,
      asserts: [
        assertVineSlotsUsage
      ]
    },
    vineExpose: {
      count: 0,
      asserts: [
        assertMacroCanOnlyHaveOneObjLiteralArg,
        assertMacroCallMustBeBare
      ]
    },
    vineOptions: {
      count: 0,
      asserts: [
        assertMacroCanOnlyHaveOneObjLiteralArg,
        assertMacroCallMustBeBare
      ]
    },
    vineCustomElement: {
      count: 0,
      asserts: [
        assertMacroCallMustBeBare
      ]
    }
  };
  let isCountCorrect = true;
  let isBareCallMacrosPass = true;
  traverse5(vineCompFn, {
    enter(node, parent) {
      if (!isVineMacroCallExpression(node)) {
        return;
      }
      const macroCalleeName = getVineMacroCalleeName(node);
      if (!macroCalleeName) {
        return;
      }
      const macroName = macroCalleeName.includes(".") ? macroCalleeName.split(".")[0] : macroCalleeName;
      if (macroCountMap[macroName]) {
        macroCountMap[macroName].count += 1;
        macroCountMap[macroName].node = node;
        macroCountMap[macroName].parent = [...parent];
      }
      const VarDeclThatMacroBeInside = parent.find((ancestor) => isVariableDeclaration4(ancestor.node));
      if (VarDeclThatMacroBeInside && BARE_CALL_MACROS.includes(macroName)) {
        isBareCallMacrosPass = false;
        vineCompilerHooks.onError(
          vineErr(
            { vineFileCtx },
            {
              msg: `\`${macroName}\` macro call is not allowed to be inside a variable declaration`,
              location: VarDeclThatMacroBeInside?.node.loc
            }
          )
        );
      }
    }
  });
  const macroMoreThanOnce = Object.entries(macroCountMap).filter(([macroName, it]) => {
    if (CAN_BE_CALLED_MULTI_TIMES_MACROS.includes(macroName)) {
      return false;
    }
    return it.count > 1;
  });
  if (macroMoreThanOnce.length > 0) {
    macroMoreThanOnce.forEach(([macroName, it]) => {
      vineCompilerHooks.onError(
        vineErr(
          { vineFileCtx },
          {
            msg: `Multiple \`${macroName}\` calls are not allowed inside Vine component function`,
            location: it.node?.loc
          }
        )
      );
    });
    isCountCorrect = false;
  }
  const isAssertsPass = Object.entries(macroCountMap).map(
    ([macroName, it]) => it.asserts.map((assert) => {
      const isPass = it.node ? assert(validatorCtx, it.node, it.parent) : true;
      vitestLogMacroAssert(macroName, isPass);
      return isPass;
    }).every(Boolean)
  ).every(Boolean);
  return isCountCorrect && isBareCallMacrosPass && isAssertsPass;
}
function validateVineModel(validatorCtx, vineCompFn) {
  const vineModelMacroCalls = [];
  _breakableTraverse(vineCompFn, (node, parent) => {
    if (isVineModel(node)) {
      vineModelMacroCalls.push({
        macroCall: node,
        parent: [...parent]
      });
    }
  });
  let isVineModelUsageCorrect = true;
  const modelCtx = {
    hasDefaultModel: false
  };
  for (const { macroCall, parent } of vineModelMacroCalls) {
    if (!assertVineModelUsage(
      validatorCtx,
      modelCtx,
      macroCall,
      parent
    )) {
      isVineModelUsageCorrect = false;
    }
  }
  return isVineModelUsageCorrect;
}
function validatePropsForSingelFC({ vineCompilerHooks, vineFileCtx }, {
  fnDeclNode: vineCompFnDecl,
  fnItselfNode
}) {
  const vineCompFnParams = fnItselfNode ? getFunctionParams(fnItselfNode) : [];
  const vineCompFnParamsLength = vineCompFnParams.length;
  let vinePropMacroCallCount = 0;
  const isCheckVinePropMacroCallPass = () => {
    let isVinePropCheckPass = true;
    traverse5(vineCompFnDecl, {
      enter(node, parent) {
        if (!isVineProp(node)) {
          return;
        }
        vinePropMacroCallCount += 1;
        const macroCalleeName = getVineMacroCalleeName(node);
        if (!macroCalleeName) {
          return;
        }
        if (macroCalleeName !== "vineProp.withDefault") {
          const typeParamsLength = node.typeParameters?.params.length;
          if (typeParamsLength !== 1) {
            vineCompilerHooks.onError(
              vineErr(
                { vineFileCtx },
                {
                  msg: `\`${macroCalleeName}\` macro call ${typeParamsLength && typeParamsLength > 1 ? "can only" : "must"} have a type parameter to specify the prop's type`,
                  location: node.loc
                }
              )
            );
            isVinePropCheckPass = false;
          }
        } else if (!node.arguments.length) {
          isVinePropCheckPass = false;
          vineCompilerHooks.onError(
            vineErr(
              { vineFileCtx },
              {
                msg: `\`${macroCalleeName}\` macro call must have at least 1 argument`,
                location: node.loc
              }
            )
          );
        } else if (node.arguments.length > 2) {
          isVinePropCheckPass = false;
          vineCompilerHooks.onError(
            vineErr(
              { vineFileCtx },
              {
                msg: `\`${macroCalleeName}\` macro call can only have at most 2 arguments`,
                location: node.loc
              }
            )
          );
        }
        const parentVarDecl = parent.find((ancestor) => isVariableDeclaration4(ancestor.node));
        const parentVarDeclarator = parent.find((ancestor) => isVariableDeclarator3(ancestor.node));
        if (!parentVarDecl || !parentVarDeclarator) {
          isVinePropCheckPass = false;
          vineCompilerHooks.onError(
            vineErr(
              { vineFileCtx },
              {
                msg: "`vineProp` macro call must be inside a `const` variable declaration",
                location: node.loc
              }
            )
          );
        } else {
          const isConst = parentVarDecl.node.kind === "const";
          if (!isConst) {
            isVinePropCheckPass = false;
            vineCompilerHooks.onError(
              vineErr(
                { vineFileCtx },
                {
                  msg: "`vineProp` macro call must be inside a `const` declaration",
                  location: parentVarDecl.node.loc
                }
              )
            );
          }
          const varDeclarator = parentVarDeclarator.node;
          if (!isIdentifier4(varDeclarator.id)) {
            isVinePropCheckPass = false;
            vineCompilerHooks.onError(
              vineErr(
                { vineFileCtx },
                {
                  msg: "the declaration of `vineProp` macro call must be an identifier",
                  location: varDeclarator.id.loc
                }
              )
            );
          }
        }
      }
    });
    return isVinePropCheckPass;
  };
  if (vineCompFnParamsLength === 0) {
    return isCheckVinePropMacroCallPass();
  } else if (vineCompFnParamsLength === 1) {
    const theOnlyFormalParam = vineCompFnParams[0];
    let isCheckFormalParamsPropPass = true;
    if (!isIdentifier4(theOnlyFormalParam)) {
      vineCompilerHooks.onError(
        vineErr(
          { vineFileCtx },
          {
            msg: "If you're defining a Vine component function's props with formal parameter, it must be one and only identifier",
            location: theOnlyFormalParam.loc
          }
        )
      );
      isCheckFormalParamsPropPass = false;
    }
    const theOnlyFormalParamTypeAnnotation = theOnlyFormalParam.typeAnnotation;
    if (!theOnlyFormalParamTypeAnnotation) {
      vineCompilerHooks.onError(
        vineErr(
          { vineFileCtx },
          {
            msg: "Vine component function's props must have a type annotation",
            location: theOnlyFormalParam.loc
          }
        )
      );
      isCheckFormalParamsPropPass = false;
    }
    if (isTSTypeAnnotation(theOnlyFormalParamTypeAnnotation) && isTSTypeLiteral2(theOnlyFormalParamTypeAnnotation.typeAnnotation)) {
      const propsTypeAnnotation = theOnlyFormalParamTypeAnnotation.typeAnnotation;
      const isTSTypeAnnotationValid = propsTypeAnnotation.members.every(
        (member) => isTSPropertySignature2(member) && (isStringLiteral3(member.key) || isIdentifier4(member.key))
      );
      if (!isTSTypeAnnotationValid) {
        vineCompilerHooks.onError(
          vineErr(
            { vineFileCtx },
            {
              msg: "Vine component function's props type annotation must be an object literal, only contains properties signature, and all properties' key must be string literal or identifier",
              location: propsTypeAnnotation.loc
            }
          )
        );
        isCheckFormalParamsPropPass = false;
      }
    } else {
      vineCompilerHooks.onError(
        vineErr(
          { vineFileCtx },
          {
            msg: "Vine component function's props type annotation must be an object literal",
            location: theOnlyFormalParamTypeAnnotation?.loc
          }
        )
      );
      isCheckFormalParamsPropPass = false;
    }
    if (vinePropMacroCallCount > 0) {
      vineCompilerHooks.onError(
        vineErr(
          { vineFileCtx },
          {
            msg: "Vine component function's props can only be defined with formal parameter or `vineProp` macro calls, not both",
            location: theOnlyFormalParam.loc
          }
        )
      );
      isCheckFormalParamsPropPass = false;
    }
    _breakableTraverse(
      vineCompFnDecl,
      (node) => {
        if (isVineProp(node)) {
          vineCompilerHooks.onError(
            vineErr(
              { vineFileCtx },
              {
                msg: "`vineProp` macro calls is not allowed when props with props formal parameter defined",
                location: node.loc
              }
            )
          );
          isCheckFormalParamsPropPass = false;
        }
      }
    );
    return isCheckFormalParamsPropPass;
  }
  vineCompilerHooks.onError(
    vineErr(
      { vineFileCtx },
      {
        msg: "Vine component function can only have one parameter",
        location: vineCompFnDecl.loc
      }
    )
  );
  return false;
}
function validateVineFunctionCompProps(validatorCtx, vineCompFnDecl) {
  const vineFnInfos = getFunctionPickedInfos(vineCompFnDecl);
  const results = vineFnInfos.map(
    (vineFnInfo) => validatePropsForSingelFC(
      validatorCtx,
      vineFnInfo
    )
  );
  return results.every(Boolean);
}
var validatesFromRoot = wrapVineValidatorWithLog([
  validateNoOutsideMacroCalls
]);
var validatesFromVineFnItself = wrapVineValidatorWithLog([
  validateVineTemplateStringUsage,
  validateMacrosUsage,
  validateVineModel
]);
var validatesFromVineFnDecl = wrapVineValidatorWithLog([
  validateVineFunctionCompProps
]);
function createVineValidator(vineCompilerHooks, vineFileCtx, vineCompFns) {
  return (validates, fromNode) => {
    return validates.map(
      (validateFn) => validateFn(
        {
          vineCompilerHooks,
          vineFileCtx,
          vineCompFns
        },
        fromNode
      )
    ).filter(Boolean).length < validates.length;
  };
}
function validateVine(compilerHooks, vineFileCtx, vineCompFnDecls) {
  const findValidateError = createVineValidator(compilerHooks, vineFileCtx, vineCompFnDecls);
  if (findValidateError(validatesFromRoot, vineFileCtx.root)) {
    return false;
  }
  let hasErrInVineCompFns = false;
  for (const vineCompFnDecl of vineCompFnDecls) {
    const vineFnInfos = getFunctionPickedInfos(vineCompFnDecl);
    vineFnInfos.forEach(({ fnDeclNode, fnItselfNode }) => {
      const hasErrInDecl = findValidateError(validatesFromVineFnDecl, fnDeclNode);
      if (hasErrInDecl) {
        hasErrInVineCompFns = true;
      }
      if (!fnItselfNode) {
        return;
      }
      const hasErrInFnItself = findValidateError(validatesFromVineFnItself, fnItselfNode);
      if (hasErrInFnItself) {
        hasErrInVineCompFns = true;
      }
    });
  }
  return !hasErrInVineCompFns;
}

// src/style/compile.ts
import postcss from "postcss";

// src/style/css-vars-plugin.ts
import MagicString2 from "magic-string";
var cssVarsPlugin = (options) => {
  const { vineCompFns } = options.fileCtx;
  let cssBindings = {};
  vineCompFns.some((fnComp) => {
    if (fnComp.scopeId === options.scopeId) {
      cssBindings = fnComp.cssBindings;
      return true;
    }
    return false;
  });
  return {
    postcssPlugin: "vine-style-css-vars",
    Once(root) {
      root.walk((ctx) => {
        if (ctx.type === "decl" && ctx.value && ctx.value.includes("v-bind")) {
          for (const cbKey in cssBindings) {
            const range = {
              start: null,
              end: null
            };
            const content = parseCssVars([ctx.value], {
              getIndex(start, end) {
                range.start = start;
                range.end = end;
              }
            });
            if (content[0] === cbKey) {
              const mg = new MagicString2(ctx.value);
              mg.overwrite(range.start, range.end, `--${cssBindings[cbKey]}`);
              mg.replaceAll("v-bind", "var");
              ctx.value = mg.toString();
              break;
            }
          }
        }
      });
    }
  };
};
cssVarsPlugin.postcss = true;

// src/style/preprocessor.ts
import merge from "merge-source-map";
var scss = (source, map, options, load = __require) => {
  const nodeSass = load("sass");
  const finalOptions = {
    ...options,
    data: source,
    sourceMap: Boolean(map)
  };
  try {
    const result = nodeSass.renderSync(finalOptions);
    const dependencies = result.stats.includedFiles;
    if (map) {
      return {
        code: result.css.toString(),
        map: merge(map, JSON.parse(result.map.toString())),
        errors: [],
        dependencies
      };
    }
    return { code: result.css.toString(), errors: [], dependencies };
  } catch (e) {
    return { code: "", errors: [e], dependencies: [] };
  }
};
var sass = (source, map, options, load = __require) => scss(
  source,
  map,
  {
    ...options,
    indentedSyntax: true
  },
  load
);
var less = (source, map, options, load = __require) => {
  const nodeLess = load("less");
  let result;
  let error = null;
  nodeLess.render(
    source,
    { ...options, syncImport: true },
    (err, output) => {
      error = err;
      result = output;
    }
  );
  if (error)
    return { code: "", errors: [error], dependencies: [] };
  const dependencies = result.imports;
  if (map) {
    return {
      code: result.css.toString(),
      map: merge(map, result.map),
      errors: [],
      dependencies
    };
  }
  return {
    code: result.css.toString(),
    errors: [],
    dependencies
  };
};
var stylus = (source, map, options, load = __require) => {
  const nodeStylus = load("stylus");
  try {
    const ref = nodeStylus(source);
    Object.keys(options).forEach((key) => ref.set(key, options[key]));
    if (map)
      ref.set("sourcemap", { inline: false, comment: false });
    const result = ref.render();
    const dependencies = ref.deps();
    if (map) {
      return {
        code: result,
        map: merge(map, ref.sourcemap),
        errors: [],
        dependencies
      };
    }
    return { code: result, errors: [], dependencies };
  } catch (e) {
    return { code: "", errors: [e], dependencies: [] };
  }
};
var processors = {
  less,
  sass,
  scss,
  stylus
};

// src/style/scoped-plugin.ts
import selectorParser from "postcss-selector-parser";
var animationNameRE = /^(?:-\w+-)?animation-name$/;
var animationRE = /^(?:-\w+-)?animation$/;
var processedRules = /* @__PURE__ */ new WeakSet();
var scopedPlugin = (options) => {
  const { id = "", fileCtx, compilerCtx } = options ?? {};
  const keyframes = /* @__PURE__ */ Object.create(null);
  const shortId = id.replace(/^data-v-/, "");
  function onPluginWarn(params) {
    compilerCtx?.vineCompileWarnings.push(
      vineWarn(fileCtx, params)
    );
  }
  function processRule(id2, rule) {
    if (processedRules.has(rule) || rule.parent && rule.parent.type === "atrule" && /-?keyframes$/.test(rule.parent.name)) {
      return;
    }
    processedRules.add(rule);
    rule.selector = selectorParser((selectorRoot) => {
      selectorRoot.each((selector) => {
        rewriteSelector(id2, selector, selectorRoot);
      });
    }).processSync(rule.selector);
  }
  function rewriteSelector(id2, selector, selectorRoot, slotted = false) {
    let node = null;
    let shouldInject = true;
    selector.each((n) => {
      if (n.type === "combinator" && (n.value === ">>>" || n.value === "/deep/")) {
        n.value = " ";
        n.spaces.before = n.spaces.after = "";
        onPluginWarn({
          msg: "the >>> and /deep/ combinators have been deprecated. Use :deep() instead.",
          location: null
        });
        return false;
      }
      if (n.type === "pseudo") {
        const { value } = n;
        if (value === ":deep" || value === "::v-deep") {
          if (n.nodes.length) {
            let last = n;
            n.nodes[0].each((ss) => {
              selector.insertAfter(last, ss);
              last = ss;
            });
            const prev = selector.at(selector.index(n) - 1);
            if (!prev || !isSpaceCombinator(prev)) {
              selector.insertAfter(
                n,
                selectorParser.combinator({
                  value: " "
                })
              );
            }
            selector.removeChild(n);
          } else {
            onPluginWarn({
              msg: "::v-deep usage as a combinator has been deprecated. Use :deep(<inner-selector>) instead.",
              location: null
            });
            const prev = selector.at(selector.index(n) - 1);
            if (prev && isSpaceCombinator(prev)) {
              selector.removeChild(prev);
            }
            selector.removeChild(n);
          }
          return false;
        }
        if (value === ":slotted" || value === "::v-slotted") {
          rewriteSelector(
            id2,
            n.nodes[0],
            selectorRoot,
            true
            /* slotted */
          );
          let last = n;
          n.nodes[0].each((ss) => {
            selector.insertAfter(last, ss);
            last = ss;
          });
          selector.removeChild(n);
          shouldInject = false;
          return false;
        }
        if (value === ":global" || value === "::v-global") {
          selectorRoot.insertAfter(selector, n.nodes[0]);
          selectorRoot.removeChild(selector);
          return false;
        }
      }
      if (n.type !== "pseudo" && n.type !== "combinator") {
        node = n;
      }
    });
    if (node) {
      node.spaces.after = "";
    } else {
      selector.first.spaces.before = "";
    }
    if (shouldInject) {
      const idToAdd = slotted ? `${id2}-s` : id2;
      selector.insertAfter(
        // If node is null it means we need to inject [id] at the start
        // insertAfter can handle `null` here
        node,
        selectorParser.attribute({
          attribute: idToAdd,
          value: idToAdd,
          raws: {},
          quoteMark: '"'
        })
      );
    }
  }
  function isSpaceCombinator(node) {
    return node.type === "combinator" && /^\s+$/.test(node.value);
  }
  return {
    postcssPlugin: "vine-style-scoped",
    Rule(rule) {
      processRule(id, rule);
    },
    AtRule(node) {
      if (/-?keyframes$/.test(node.name) && !node.params.endsWith(`-${shortId}`)) {
        keyframes[node.params] = node.params = `${node.params}-${shortId}`;
      }
    },
    OnceExit(root) {
      if (Object.keys(keyframes).length) {
        root.walkDecls((decl) => {
          if (animationNameRE.test(decl.prop)) {
            decl.value = decl.value.split(",").map((v) => keyframes[v.trim()] || v.trim()).join(",");
          }
          if (animationRE.test(decl.prop)) {
            decl.value = decl.value.split(",").map((v) => {
              const vals = v.trim().split(/\s+/);
              const i = vals.findIndex((val) => keyframes[val]);
              if (i !== -1) {
                vals.splice(i, 1, keyframes[vals[i]]);
                return vals.join(" ");
              } else {
                return v;
              }
            }).join(",");
          }
        });
      }
    }
  };
};
scopedPlugin.postcss = true;

// src/style/trim-plugin.ts
var trimPlugin = () => {
  return {
    postcssPlugin: "vine-style-trim",
    Once(root) {
      root.walk(({ type, raws }) => {
        if (type === "rule" || type === "atrule") {
          if (raws.before)
            raws.before = "\n";
          if ("after" in raws && raws.after)
            raws.after = "\n";
        }
      });
    }
  };
};
trimPlugin.postcss = true;

// src/style/compile.ts
async function compileVineStyle(compilerCtx, params) {
  let { source } = params;
  const {
    vineFileId,
    isScoped,
    scopeId,
    preprocessLang,
    inputSourceMap,
    preprocessOptions = compilerCtx.options.preprocessOptions ?? {},
    postcssOptions = compilerCtx.options.postcssOptions,
    postcssPlugins: userPostcssPlugins = compilerCtx.options.postcssPlugins ?? []
  } = params;
  const preprocessor = preprocessLang && processors[preprocessLang];
  const preProcessedSource = preprocessor?.(
    source,
    inputSourceMap,
    preprocessOptions
  );
  const map = preProcessedSource ? preProcessedSource.map : inputSourceMap;
  if (preProcessedSource) {
    source = preProcessedSource.code;
  }
  const postcssPlugins = [
    trimPlugin(),
    ...userPostcssPlugins
  ];
  if (isScoped) {
    const shortId = scopeId.replace(/^data-v-/, "");
    const longId = `data-v-${shortId}`;
    postcssPlugins.push(
      scopedPlugin({
        compilerCtx,
        id: longId,
        fileCtx: compilerCtx.fileCtxMap.get(vineFileId)
      })
    );
  }
  if (source.includes("v-bind")) {
    postcssPlugins.push(cssVarsPlugin({
      fileCtx: compilerCtx.fileCtxMap.get(vineFileId),
      scopeId
    }));
  }
  const postCSSOptions = {
    ...postcssOptions,
    // prevent warnings, we don't need file name
    from: void 0,
    to: void 0
  };
  if (map) {
    postCSSOptions.map = {
      inline: false,
      annotation: false,
      prev: map
    };
  }
  const errors = [];
  if (preProcessedSource && preProcessedSource.errors.length) {
    errors.push(...preProcessedSource.errors);
  }
  const dependencies = new Set(
    preProcessedSource ? preProcessedSource.dependencies : []
  );
  const recordPlainCssDependencies = (messages) => {
    messages.forEach((msg) => {
      if (msg.type === "dependency") {
        dependencies.add(msg.file);
      }
    });
    return dependencies;
  };
  const result = postcss(postcssPlugins).process(source, postCSSOptions);
  return result.then((result2) => ({
    code: result2.css || "",
    map: result2.map && result2.map.toJSON(),
    errors,
    rawResult: result2,
    dependencies: recordPlainCssDependencies(result2.messages)
  })).catch((error) => ({
    code: "",
    map: void 0,
    errors: [...errors, error],
    rawResult: void 0,
    dependencies
  }));
}

// src/index.ts
function createCompilerCtx(options = {}) {
  return {
    fileCtxMap: /* @__PURE__ */ new Map(),
    vineCompileErrors: [],
    vineCompileWarnings: [],
    isRunningHMR: false,
    options: {
      inlineTemplate: true,
      // default inline template
      // Maybe some default options ...
      ...options
    }
  };
}
function createVineFileCtx(code, fileId, vineCompileCtx) {
  const { fileCtxCache, babelParseOptions = {} } = vineCompileCtx;
  const root = babelParse(code, babelParseOptions);
  const vineFileCtx = {
    root,
    fileId,
    renderOnly: fileCtxCache ? fileCtxCache.renderOnly : false,
    hmrCompFnsName: fileCtxCache ? fileCtxCache.hmrCompFnsName : null,
    isCRLF: code.includes("\r\n"),
    fileMagicCode: new MagicString3(code),
    vineCompFns: [],
    userImports: {},
    styleDefine: {},
    vueImportAliases: {},
    get originCode() {
      return this.fileMagicCode.original;
    },
    getAstNodeContent(node) {
      return this.originCode.slice(node.start, node.end);
    },
    getLinkedTSTypeLiteralNodeContent(node) {
      return `{ ${node.members.map((member) => {
        const memberOriginCode = this.originCode.slice(member.start, member.end);
        return `${member.type === "TSMethodSignature" || member.type === "TSPropertySignature" ? `${createLinkedCodeTag("left", this.getAstNodeContent(member.key).length)}${memberOriginCode}` : memberOriginCode}`;
      }).join(", ")} }`;
    }
  };
  return vineFileCtx;
}
function doValidateVine(vineCompilerHooks, vineFileCtx, vineCompFnDecls) {
  validateVine(vineCompilerHooks, vineFileCtx, vineCompFnDecls);
  vineCompilerHooks.onValidateEnd?.();
}
function doAnalyzeVine(vineCompilerHooks, vineFileCtx, vineCompFnDecls) {
  analyzeVine(vineCompilerHooks, vineFileCtx, vineCompFnDecls);
  vineCompilerHooks.onAnalysisEnd?.();
}
function compileVineTypeScriptFile(code, fileId, vineCompileCtx, ssr = false) {
  const { compilerHooks } = vineCompileCtx;
  const compilerOptions = compilerHooks.getCompilerCtx().options;
  const vineFileCtx = createVineFileCtx(code, fileId, vineCompileCtx);
  compilerHooks.onBindFileCtx?.(fileId, vineFileCtx);
  const vineCompFnDecls = findVineCompFnDecls(vineFileCtx.root);
  doValidateVine(compilerHooks, vineFileCtx, vineCompFnDecls);
  doAnalyzeVine(compilerHooks, vineFileCtx, vineCompFnDecls);
  transformFile(
    vineFileCtx,
    compilerHooks,
    compilerOptions?.inlineTemplate ?? true,
    ssr
  );
  compilerHooks.onEnd?.();
  return vineFileCtx;
}
export {
  VineBindingTypes,
  _breakableTraverse,
  compileVineStyle,
  compileVineTypeScriptFile,
  createCompilerCtx,
  createVineFileCtx,
  doAnalyzeVine,
  doValidateVine,
  exitTraverse,
  findVineCompFnDecls,
  isComponentNode2 as isComponentNode,
  isElementNode,
  walkVueTemplateAst
};
